// bme.c  
#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include "pico/stdlib.h"
#include "math.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "hardware/uart.h"
#include "bme.h"  // mine       

#define LOG 0
#define DISPLAY_SECS 8
//=============================================
void do_bme280()
{
   unsigned int display_count;
   uint16_t n;

   read_bme280();
   compensate_temperature();     // sensor.temperature_F = temperature in Fahrenheit
   compensate_pressure();        // sensor.pressure = pressure in mb
   compensate_humidity();        // sensor.humidity in %

   while(!zda_flag);    // wait for a ZDA message
   show_zda_buffer(); 
   decode_ZDA(1);
// display_utc_lcd();
   zda_flag = 0;

   while(!gga_flag);    // wait for a GGA message
   show_gga_buffer(); 
   decode_GGA(1);
   gga_flag = 0;

   make_wx_entry(&flash_entry);  // make a log entry from sensor.temperature_F, p, sensor.humidity 
   show_entry(&flash_entry);

   uart_puts(UART_ID, "Keyboard Commands (Upper or Lower Case)\n");
   uart_puts(UART_ID, "   V: Show Version\n");
   uart_puts(UART_ID, "   B: Show BME-280\n");
   uart_puts(UART_ID, "   O: Log BME-280 Once\n");
   uart_puts(UART_ID, "   L: Log BME-280 Every 5 minutes\n");
   uart_puts(UART_ID, "   X: Stop Logging BME-280\n");
   uart_puts(UART_ID, "   D: Dump All of the Flash Logs to uart\n");
   uart_puts(UART_ID, "   E: Erase Data Flash?  (then Y/N)\n"); 
   uart_puts(UART_ID, "   S: Reset Start Date?  (then Y/N)\n");
   uart_puts(UART_ID, "   P: BME-280 Parameters\n");
   uart_puts(UART_ID, "   R: Reset Barometric Reference to 29.92\n");
   uart_puts(UART_ID, "   F: Set Default Local Offset and DST\n");
   uart_puts(UART_ID, "   H: Repeat This List\n\n"); 

   uart_puts(UART_ID, "> Ready For Commands\n\n"); 
//-----------------------
// Main Loop ffff
   display_count = DISPLAY_SECS;
   sleep_ms(500);

   if(logging)  
   {
      bme_log_count = 10;     // for a fast start
      bme_log_flag = 1;
      ts_number = 0;        // time stamp number
      uart_puts(UART_ID, "Logging every 5 minutes. Press X to stop\n\n"); 
   }    
   else  {bme_log_flag = 0; }

   while(1)      // This is the loop for the main program
   {
      if(secs_flag)
      {
         if( !gpio_get(PB1) )   { show_sats_leap();}   // Just show #sats and Leap Seconds while PB1 is pressed
         read_bme280();
         compensate_temperature();     // sensor.temperature_F = temperature in Fahrenheit
         compensate_pressure();        // sensor.pressure = pressure in mb
         compensate_humidity();        // sensor.humidity in %

         display_count--;

         if(display_count == 0)    // for lcd
         {
            gpio_put(LED1,1);
            show_wx_data_lcd();     // show wx data on lcd
            display_count = DISPLAY_SECS; 
         }
         else { gpio_put(LED1,0); }

         if(display_count == 4)  { lcd_altitude_feet(); }

         if( --bme_log_count == 0 )    // for logging bme in Flash
         { 
            if( bme_log_flag )    // log it in flash
            {
               read_bme280();
               compensate_temperature();     // sensor.temperature_F = temperature in Fahrenheit
               compensate_pressure();        // sensor.pressure = pressure in mb
               compensate_humidity();        // sensor.humidity in %

               while(!zda_flag);
               show_zda_buffer(); 
               decode_ZDA(1);

               // display_utc_lcd();
               zda_flag = 0;

               while(!gga_flag);
               show_gga_buffer(); 
               decode_GGA(1);
               gga_flag = 0;

               make_wx_entry(&flash_entry); // make a log entry from sensor.temperature_F, p, sensor.humidity 

               // Write it to Flash Log
               for(n = 0; n<256; n++)  { temp_buffer[n] = 0xFF; }
               memcpy(temp_buffer,&flash_entry, structlogsize);
               write_page(Page_Index, temp_buffer);
               show_log_page(Page_Index);
               Page_Index++;
            }
            
            bme_log_count = 295;  // 5 minutes (300 seconds minus some time to read the GPS messages) 
         }
         secs_flag = 0;
      
      } // end secs
     //--------------------------------    
     // Still in while(1)
     do_uart_RX();    // Use uart RX to decide what to send to uart

   }  // end while

}  // end function
//==============================================
// Use uart RX to decide what to send to uart
void do_uart_RX()
{
   uint16_t n;
   uint32_t page_end;

   if( uart_is_readable(UART_ID) )   
   { 
      uart_char = uart_getc(UART_ID) & (~0x20);    // read uart FIFO (upper or lower case)

      switch(uart_char)
      {
         case 'W':   // Set Local Offset and DST to Default
            show_stuff();    // show some stuff
            sleep_ms(4000);
            break;
         //----------
         case 'F':   // Set Local Offset and DST to Default
            do_local(-8, 1, 1);    // default local offset, default dst On, erase
            sleep_ms(4000);
            break;
         //----------
         case 'T':   // Show UTC and Local Time
            while(!zda_flag);
            show_zda_buffer(); 
            decode_ZDA(1);
            display_utc_uart();
            zda_flag = 0;

            get_local();
            sleep_ms(4000);
            break;
         //----------
         case 'O':   // Log Once
            read_bme280();
            compensate_temperature();     // sensor.temperature_F = temperature in Fahrenheit
            compensate_pressure();        // sensor.pressure = pressure in mb
            compensate_humidity();        // sensor.humidity in %

            while(!zda_flag);
            show_zda_buffer(); 
            decode_ZDA(1);
            // display_utc_lcd();
            zda_flag = 0;

            while(!gga_flag);
            show_gga_buffer(); 
            decode_GGA(1);
            gga_flag = 0;

            make_wx_entry(&flash_entry); // make a log entry from sensor.temperature_F, p, Humidity 
       
            for(n = 0; n<256; n++)  { temp_buffer[n] = 0xFF; }
            memcpy(temp_buffer,&flash_entry, structlogsize);
            write_page(Page_Index, temp_buffer);
            show_log_page(Page_Index);
            Page_Index++;
            break;
         //----------
         case 'V':   // Version message
            strcpy(line, version);
            strcat(line,"Start Date: ");
            strcat(line, start_date);
            uart_puts(UART_ID, line); 
            break;
         //----------
         case 'B':   // Show BME-280
            read_bme280();
            compensate_temperature();     // sensor.temperature_F = temperature in Fahrenheit
            compensate_pressure();        // sensor.pressure = pressure in mb
            compensate_humidity();        // sensor.humidity in %

            uart_puts(UART_ID, pressure_altitude_str);

            while(!zda_flag);
            show_zda_buffer(); 
            decode_ZDA(1);
            // display_utc_lcd();
           zda_flag = 0;

            while(!gga_flag);
            show_gga_buffer(); 
            decode_GGA(1);
            gga_flag = 0;

            make_wx_entry(&flash_entry); // make a log entry from sensor.temperature_F, p, Humidity 
            show_entry(&flash_entry);
            break;
         //----------
          case 'L':  // Log Every 5 minutes
            bme_log_count = 10;     // for a fast start
            bme_log_flag = 1;
            ts_number = 0;        // time stamp number
            uart_puts(UART_ID, "Logging every 5 minutes. Press X to stop\n\n"); 
            break;
         //----------
          case 'X':  // Stop Logging BME
            bme_log_flag = 0;
            uart_puts(UART_ID, "Logging Stopped\n\n"); 
            break;
         //----------
          case 'D':    // Dump All of the Flash logs to uart
            uart_puts(UART_ID,"\nFull Flash Log\n");      
            page_end = Page_Index - 1;
            erase_flag = 0;
            dump_flash(0, page_end);
            break;
          //----------
         case 'E':    // Erase Flash Data
            uart_puts(UART_ID, "\nErase Data Flash?   Y/N\n"); 
            erase_flag = 1;
            break;

         case 'S':    // Reset Start Date
            uart_puts(UART_ID, "\nReset Start Date?  Y/N\n\n");
            reset_rx_start = 1;
            break;

         case 'Y':    // Yes
            if(erase_flag)  
            {
               erase_all_sectors();
               uart_puts(UART_ID, "\nFlash Data Erased\n\n"); 
            }
            erase_flag = 0;

            if(reset_rx_start)  { do_start_date(1); }  // Erase Start Date in Flash. write today's date
            reset_rx_start = 0;
            break;

         case 'N':    // No
            erase_flag = 0;
            reset_rx_start = 0;
            uart_puts(UART_ID, "\nFlash Data Not Erased\n\n"); 
            break;
         //----------
         case 'P':    // BME-280 Parameters
            uart_puts(UART_ID, "\nBME-280 Parameters\n\n");
            bme280_parameters();
            break;
         //----------
         case 'R':    // Reset Barometric Reference to 29.92
            uart_puts(UART_ID, "\nReset Barometric Reference to 29.92\n\n");
            erase_bref_flash();
            do_B_reference(0x2992);
            sleep_ms(100);
            break;
         //----------
         case 'H':    // Repeat this List
            uart_puts(UART_ID, "Keyboard Commands (Upper or Lower Case)\n");
            uart_puts(UART_ID, "   V: Show Version\n");
            uart_puts(UART_ID, "   B: Show BME-280\n");
            uart_puts(UART_ID, "   O: Log BME-280 Once\n");
            uart_puts(UART_ID, "   L: Log BME-280 Every 5 minutes\n");
            uart_puts(UART_ID, "   X: Stop Logging BME-280\n");
            uart_puts(UART_ID, "   D: Dump All of the Flash Logs to uart\n");
            uart_puts(UART_ID, "   E: Erase Data Flash?  (then Y/N)\n"); 
            uart_puts(UART_ID, "   S: Reset Start Date?  (then Y/N)\n");
            uart_puts(UART_ID, "   P: BME-280 Parameters\n");
            uart_puts(UART_ID, "   R: Reset Barometric Reference to 29.92\n");
            uart_puts(UART_ID, "   F: Set Default Local Offset and DST\n");
            uart_puts(UART_ID, "   H: Repeat This List\n\n"); 
            break;
         //----------
         default:
            erase_flag = 0;
 
      }  // end switch
   }  // end of if uart was readable
} // end uart RX
//====================================================
// Read the BME280 data for pressure, temperature, and humidity
void read_bme280(void)
{
// Get the raw bme280 data
/*  bmedata
press_msb 0xF7 bmedata[0]       press_lsb 0xF8 bmedata[1]      press_xlsb 0xF9  bmedata[2]
       msb                              lsb                            xlsb  0000
       7-0                              7-0                            7-4   0000 
      23-16                            15-8                            7-4   3-0

temp_msb  0xFA  bmedata[3]     temp_lsb  0xFB  bmedata[4]      tmp_xlsb 0xFC bmedata[5]
       msb                              lsb                            xlsb  0000
       7-0                              7-0                            7-4   0000 
      23-16                            15-8                            7-4    3-0

hum_msb 0xFD bmedata[6]      hum_lsb 0xFE bmedata[7] 
         15-8                         7 - 0
*/
   unsigned char address;

   address = bme_ctrl_meas;
   i2c_write_byte_bme(address,0x2D);  // In forced mode it must be called for every reading
   sleep_ms(5);

   i2c_outbuffer[0] = press_msb;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, bmedata, 8, false);

// Pressure - It is done this way so shifting a signed number doesn't screw it up
   adc_p = bmedata[0];
   adc_p = adc_p<<8;
   adc_p = adc_p | bmedata[1];
   adc_p = adc_p<<8;
   adc_p = adc_p | bmedata[2];
   adc_p = adc_p>>4;

// Temperature
   adc_t = bmedata[3];
   adc_t = adc_t<<8;
   adc_t = adc_t | bmedata[4];
   adc_t = adc_t<<8;
   adc_t = adc_t | bmedata[5];
   adc_t = adc_t>>4;

// Humidity
   adc_h = bmedata[6]<<8 | bmedata[7]; 
   asm(" nop");    // for debugger
}
//===================================================
void bme_init()
{
    unsigned char address;

// BME280 reset
    address = bme_reset;
    i2c_write_byte_bme(address,0x00);
    sleep_us(100);

// Initialize the BME-280
// config    d7   d6   d5 | d4   d3   d2 | d1 | d0
//              t_sb          filter        x   spi3W_en
//            0    0    0    0    0    0    0    0
// t_sb = 000 (0.5 ms  standby time)
// filter = 000  filter off
// spi3W_en = 0 for 4-wire
    address = bme_config;
    i2c_write_byte_bme(address,0x00);
    sleep_us(100);

// humidity: oversampling x1   ('0' = sensor off)
    i2c_write_byte_bme(0xF2,0x01);
    sleep_us(100);

// ctrl_meas    d7   d6   d5 |  d4   d3   d2 | d1   d0
//                 osrs_t           osrs_p       mode
//               0    0    1     0    1    1   0     1
// osrs_t = 001  oversampling x 1 17-bit  ('0' = sensor off)
// osrs_p = 011 oversampling x4   ('0' = off)
// mode = 11 (normal)  (0 = sleep; 01 and 10 = Forced Mode)
   address = bme_ctrl_meas;
   i2c_write_byte_bme(address,0x2D);  // In forced mode it must be called for every reading
   sleep_us(100);

   wait_ready();
}
//=================================================
// Read one i2C byte from bme
unsigned char i2c_read_byte(unsigned char adr)
{
   i2c_outbuffer[0] = adr;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, i2c_inbuffer, 1, false);
   return(i2c_inbuffer[0]);
}
//==================================
// Read i2C bytes from bme
void i2c_read(unsigned char adr, unsigned int len)
{
   i2c_outbuffer[0] = adr;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, i2c_inbuffer, len, false);
}
//=======================================================
// Write one i2C byte to bme
void i2c_write_byte_bme(unsigned char adr, unsigned char data)
{
   i2c_outbuffer[0] = adr;  // register address
   i2c_outbuffer[1] = data;  // register data

   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 2, false);
}
//==================================
uint16_t wait_ready()
{
    int n, timeout;
    unsigned char busy, data;

    busy = 1;
    n = 1000;
    timeout = 0;

    while(busy)
    {
       data = i2c_read_byte(0xF3);
       if( !(data & 0x08)) {busy = 0;}
       if(n == 0)  {timeout = 1;}
       n--;
    }
    return timeout;
}
//==================================
// Get the calibration values
void getcal()
{
    // cal data for pressure and temperature
    i2c_outbuffer[0] = 0x88;  // register address
    i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
    i2c_read_blocking(i2c_default, BME_ID, caldata, 24, false);  // &&&& was 28

   // Humidity
   wait_ready();
   humdata[0]= i2c_read_byte (0xA1);   // for dig_H1 (unsigned char)

   wait_ready();
   i2c_outbuffer[0] = 0xE1;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, humdata+1, 7, false);  // humdata[1] to humdata[8]

// Temperature
// Make digs for compensation functions. All digs are global
// Requires that the caldata array be unsigned

   // caldata[0] = dig_T1 lsb; caldata[1] = dig_T1 msb unsigned
   dig_T1 = caldata[1]<<8 | caldata[0];

   // caldata[2] = dig_T2 lsb; caldata[3] = dig_T2 msb signed
   dig_T2 = caldata[3]<<8 | caldata[2];

   // caldata[4] = dig_T3 lsb; caldata[5] = dig_T3 msb signed
   dig_T3 = caldata[5]<<8 | caldata[4];

// Pressure
   // caldata[6] = dig_P1 lsb; caldata[7] = dig_P1 msb unsigned
   dig_P1 = caldata[7]<<8 | caldata[6];

   // caldata[8] = dig_P2 lsb; caldata[9] = dig_P2 msb signed
   dig_P2 = caldata[9]<<8 | caldata[8];

   // caldata[10] = dig_P3 lsb; caldata[11] = dig_P3 msb    signed
   dig_P3 = caldata[11]<<8 | caldata[10];

   // caldata[12] = dig_P4 lsb; caldata[13] = dig_P4 msb    signed
   dig_P4 = caldata[13]<<8 | caldata[12];

   // caldata[14] = dig_P5 lsb; caldata[15] = dig_P5 msb    signed
   dig_P5 = caldata[15]<<8 | caldata[14];

   // caldata[16] = dig_P6 lsb; caldata[17] = dig_P6 msb    signed
   dig_P6 = caldata[17]<<8 | caldata[16];

   // caldata[18] = dig_P7 lsb; caldata[19] = dig_P7 msb    signed
   dig_P7 = caldata[19]<<8 | caldata[18];

   // caldata[20] = dig_P8 lsb; caldata[21] = dig_P8 msb    signed
   dig_P8 = caldata[21]<<8 | caldata[20];

   // caldata[22] = dig_P9 lsb; caldata[23] = dig_P9 msb    signed
   dig_P9 = caldata[23]<<8 | caldata[22];

   wait_ready();

// make the humidity digs
   // humdata[0] is dig_H3 7:0 (unsigned char);
   dig_H1 = humdata[0];  //  7:0

   // humdata[1] is dig_H2 7:0; humdata[2] is dig_H2 8:15
   dig_H2 = (humdata[2]<<8) | humdata[1];

   // humdata[3] is dig_H3 7:0  (unsigned char);
   dig_H3 = humdata[3];   // dig_H3 7:0

   // humdata[4] (7:0) is dig_H4 11"4; humdata[5] (3:0) is dig_H4 3:0
   dig_H4 = (humdata[4] <<4) | (humdata[5] & 0x0F);

   // humdata[5] 7:4 is also dig_H5 3:0; humdata[6] is dig_H5 11:4
   dig_H5 = ( (humdata[5]>>4) & 0x0F) | (humdata[6]<<4);

   dig_H6 = humdata[7];     // signed char
}
//=========================================
// Convert Binary signed temperature to BCD; 4 digits
// temperature from bme280 is in Celsius 
void temperature_itoa(char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3;
   unsigned char char1, char2, char3, char4;

   int i;
   unsigned char hcarry;
   unsigned int bin, sign;

// 0x08DD = 2269.; 2269/10 = 226; 226 * 18 = 4084; 4084 + 3200 = 7284 = 72.8 deg F;  
// 0xF723 = -2269.; -2269/10 = -226; 226 * 18 = -4084; -4084 + 3200 = -884 = -8.8 deg F;  

   if(sensor.temperature_F < 0)
   {
      bin = -sensor.temperature_F;
      sign = 1; 
   }
   else
   { 
       bin = sensor.temperature_F;
       sign = 0;
   }

// Convert to BCD
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

/*
// Make the string; do not remove leading zeros
   if(sign)  
   { 
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = '.';
      str[5] = char4;
      str[6] = 0;
   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = '.';
      str[4] = char4;
      str[5] = 0;
   }
*/

// Make the string; remove leading zeros
    if(sign)    // number is negative
    { 
       str[0] = '-';

       if(char1 != 0x30)
       { 
          str[1] = char1;
          str[2] = char2;
          str[3] = char3;
          str[4] = '.';
          str[5] = char4;
          str[6] = 0;
          for(i=0; i<7; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[1] = char3;
      str[2] = '.';
      str[3] = char4;
      str[4] = 0;
      for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}   // for displaying on lcd later
    }  // end of sign is negative
    else   // sign is positive
    {
       if(char1 != 0x30)
       { 
          str[0] = char1;
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[0] = char2;
          str[1] = char3;
          str[2] = '.';
          str[3] = char4;
          str[4] = 0;
          for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[0] = char3;
      str[1] = '.';
      str[2] = char4;
      str[3] = 0;
      for(i=0; i<4; i++)  {temperature_lcd[i] = str[i];}    // for displaying on lcd later
    }  // end of sign is positive

}
//==========================================================
void show_wx_data_lcd()
{
   int n;

   temperature_itoa(nstr);   // remove leading zeros; end with NULL
   pressure_ltoa(nstr);      // remove leading zeros; end with NULL
   humidity_btoa(nstr);

   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen

// Write Temperature
// T:
// Temperature
   lcd_wr_instr(0x80);  // position
   lcd_wr_data('T');
   lcd_wr_data(':');
   lcd_wr_data(' ');
   for(n=0; n<10; n++)
   {
      if(temperature_lcd[n] == 0)  {break;}   // NULL character
      lcd_wr_data(temperature_lcd[n]);
   }
   lcd_wr_data(' ');
   lcd_wr_data('F');

// H:
// Humidity
   lcd_wr_instr(0x8B);  // position
   lcd_wr_data('H');
   lcd_wr_data(':');
//   lcd_wr_data(' ');
   for(n=0; n<2; n++)
   {
      if(humidity_lcd[n] == 0)  {break;}   // NULL character
      lcd_wr_data(humidity_lcd[n]);
   }
//   lcd_wr_data(' ');
   lcd_wr_data('%');

// P:
// Pressure
   lcd_wr_instr(0xC0);  // position
   lcd_wr_data('P');
   lcd_wr_data(':');
   lcd_wr_data(' ');
   for(n=0; n<10; n++)
   {
      if(pressure_lcd[n] == 0)  {break;}   // NULL character
      lcd_wr_data(pressure_lcd[n]);
   }
   lcd_wr_data(' ');
   lcd_wr_data('m');
   lcd_wr_data('b');
}
//====================================================
void log_wx_data_ts()
{
   log_time();   // time stamp in line

   strcat(line,"Temperature: ");
   temperature_itoa(nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," Degrees    ");

   strcat(line,"Pressure: ");
   pressure_ltoa(nstr);        // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb    ");

   strcat(line,"Humidity: ");
   humidity_btoa(nstr);
   strcat(line,nstr);

   strcat(line," %\r\n\n");

   uart_puts(UART_ID,line);  
}
//===========================================================
void log_wx_devid_ts()
{
   strcpy(line,"Device ID: ");
   devid(nstr);
   strcat(line,nstr);
   strcat(line,"\r\n\n");

   uart_puts(UART_ID,line);  
}
//===========================================================
// Convert hex pressure to BCD; 6 digits
void pressure_ltoa(char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3, bcd4, bcd5;
   unsigned char char1, char2, char3, char4, char5;

   int i;
   unsigned char hcarry;
   unsigned long bin;

   bin = sensor.pressure;

// Adapted from 8-bit microcontroller code using a DAA instruction
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;
   bcd4 = 0;
   bcd5 = 0;

   for(i=0; i<24; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x800000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD4
      bcd4 = bcd4<<1;
      bcd4 = bcd4 + hcarry;

      if(bcd4 >9)
      {
         bcd4 = bcd4 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD5
      bcd5 = bcd5<<1;
      bcd5 = bcd5 + hcarry;

      if(bcd5 >9)
      {
         bcd5 = bcd5 - 10;
//         hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd5 + 0x30;
   char2 = bcd4 + 0x30;
   char3 = bcd3 + 0x30;
   char4 = bcd2 + 0x30;
   char5 = bcd1 + 0x30;

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = '.';
      str[5] = char5;
      str[6] = 0;      // NULL
      for(i=0; i<7; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }
   
   if(char2 != 0x30)
   { 
      str[0] = char2;
      str[1] = char3;
      str[2] = char4;
      str[3] = '.';
      str[4] = char5;
      str[5] = 0;    // NULL
      for(i=0; i<6; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   if(char3 != 0x30)
   { 
      str[0] = char3;
      str[1] = char4;
      str[2] = '.';
      str[3] = char5;
      str[4] = 0;     // NULL
      for(i=0; i<5; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   str[0] = char4;
   str[1] = '.';
   str[2] = char5;
   str[3] = 0;     // NULL
   for(i=0; i<4; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd

   asm(" nop");    // for emulator
}
//=============================================
// Convert Binary signed humidity to BCD - 4 digits
void humidity_btoa(char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1;
   unsigned char char1, char2;

   int i;
   unsigned char hcarry;
   unsigned char bin;

   bin = sensor.humidity;

   if(bin>99)  bin=99;   // maximum number

   bcd0 = 0;
   bcd1 = 0;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
//         hcarry = 1;
      }
       else {hcarry = 0;}
   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = 0;
      for(i=0; i<3; i++)  {humidity_lcd[i] = str[i];}
   }
   else
   {
      str[0] = char2;
      str[1] = 0;
      for(i=0; i<2; i++)  {humidity_lcd[i] = str[i];}  // for displaying on lcd
   }  
}
//==================================================
void itoa_hex(unsigned char bin)
{
   unsigned char1, char2, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}
    
// Add to string
   nstr[0] = char1;
   nstr[1] = char2;
   nstr[2] = 0;  // NULL
}
//=========================================
void itoa_hex_int(unsigned int bin)
{
   unsigned char cdigit[8], m, n;
   unsigned int temp;

//   bin = 0x1234;  // for testing
   m = 3;

// Convert to ascii
   for(n=0; n<4; n++)
   {
     temp = bin & 0x0F;
     if(temp<10)  {cdigit[n] = temp + 0x30;}
     else {cdigit[n] = temp + 0x37;}
     bin = bin>>4;
     nstr[m--] = cdigit[n];
   }

   nstr[n] = 0;   // NULL
}
//=========================================
void itoa_hex_long(unsigned long bin)
{
   unsigned char cdigit[8], m, n;
   unsigned long temp;

//   bin = 0x12345678;   // for testing
   m = 7;
// Convert to ascii
   for(n=0; n<8; n++)
   {
     temp = bin & 0x0F;
     if(temp<10)  {cdigit[n] = temp + 0x30;}
     else {cdigit[n] = temp + 0x37;}
     bin = bin>>4;
     nstr[m--] = cdigit[n];
   }

   nstr[n] = 0;   // NULL
}
//=========================================
void log_cal()
{
   unsigned char k;

   strcpy(line,"Cal Data 0x88 to 0x8F (24 bytes)\r\n");

   for(k = 0; k<24; k++)
   {
      itoa_hex(caldata[k]);
      strcat(line,nstr);
      strcat(line,"   ");
   }

   strcat(line,"\nCal Hum (8 bytes)   ");
 
   for(k = 0; k<8; k++)
   {
      itoa_hex(humdata[k]);
      strcat(line,nstr);
      strcat(line,"   ");
   }

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}

/*
unsigned short dig_T1, dig_P1;
long int dig_T2, dig_T3, dig_P2, dig_P3, dig_P4, dig_P5, dig_P6, dig_P7, dig_P8, dig_P9;
unsigned char dig_H1, dig_H3;
int dig_H2, dig_H4, dig_H5;
char dig_H6;
*/
//=======================================
void log_digs()
{
   strcpy(line,"dig T1: ");
   itoa_hex_int(dig_T1);
   strcat(line,nstr);

   strcat(line,"   dig T2: ");
   itoa_hex_int(dig_T2);
   strcat(line,nstr);

   strcat(line,"   dig T3: ");
   itoa_hex_int(dig_T3);
   strcat(line,nstr);

   strcat(line,"\n\ndig P1: ");
   itoa_hex_int(dig_P1);
   strcat(line,nstr);

   strcat(line,"    dig P2: ");
   itoa_hex_int(dig_P2);
   strcat(line,nstr);

   strcat(line,"    dig P3: ");
   itoa_hex_int(dig_P3);
   strcat(line,nstr);

   strcat(line,"    dig P4: ");
   itoa_hex_int(dig_P4);
   strcat(line,nstr);

   strcat(line,"\ndig P5: ");
   itoa_hex_int(dig_P5);
   strcat(line,nstr);

   strcat(line,"    dig P6: ");
   itoa_hex_int(dig_P6);
   strcat(line,nstr);

   strcat(line,"    dig P7: ");
   itoa_hex_int(dig_P7);
   strcat(line,nstr);

   strcat(line,"    dig P8: ");
   itoa_hex_int(dig_P8);
   strcat(line,nstr);

   strcat(line,"    dig P9: ");
   itoa_hex_int(dig_P9);
   strcat(line,nstr);

   strcat(line,"\n\ndig H1: ");
   itoa_hex(dig_H1);
   strcat(line,nstr);

   strcat(line,"    dig H2: ");
   itoa_hex_int(dig_H2);
   strcat(line,nstr);

   strcat(line,"    dig H3: ");
   itoa_hex(dig_H3);
   strcat(line,nstr);

   strcat(line,"    dig H4: ");
   itoa_hex_int(dig_H4);
   strcat(line,nstr);

   strcat(line,"    dig H5: ");
   itoa_hex_int(dig_H5);
   strcat(line,nstr);

   strcat(line,"    dig H6: ");
   itoa_hex(dig_H6);
   strcat(line,nstr);

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}
//===============================================
void log_raw()
{
   strcpy(line,"raw data:\n");

   strcat(line,"pres_msb: ");
   itoa_hex(bmedata[0]);
   strcat(line,nstr);

   strcat(line,"    pres_lsb: ");
   itoa_hex(bmedata[1]);
   strcat(line,nstr);

   strcat(line,"    pres_xlsb: ");
   itoa_hex(bmedata[2]);
   strcat(line,nstr);

   strcat(line,"\ntemp_msb: ");
   itoa_hex(bmedata[3]);
   strcat(line,nstr);

   strcat(line,"   temp_lsb: ");
   itoa_hex(bmedata[4]);
   strcat(line,nstr);

   strcat(line,"   temp_xlsb: ");
   itoa_hex(bmedata[5]);
   strcat(line,nstr);

   strcat(line,"\nhum_msb: ");
   itoa_hex(bmedata[6]);
   strcat(line,nstr);

   strcat(line,"   hum_lsb:");
   itoa_hex(bmedata[7]);
   strcat(line,nstr);

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  

   strcpy(line,"adc_p: ");      // unsigned
   itoa_hex_long(adc_p);
   strcat(line,nstr);

   strcat(line,"    adc_t: ");  // signed
   itoa_hex_long(adc_t);
   strcat(line,nstr);

   strcat(line,"    adc_h: ");  // unsigned
   itoa_hex_long(adc_h);
   strcat(line,nstr);

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}
//==============================================
void log_vars()
{
   unsigned int n;

   strcpy(line,"variables:   ");

   for(n=0; n<5; n++)
   {
      itoa_hex_long(testvars[n]);
      strcat(line,nstr);

      if(n == 7)  { strcat(line,"\n"); } 
      else { strcat(line,"   "); }
   }

   strcat(line,"\r\n\n");
   uart_puts(UART_ID,line);  
}
//==========================================================
// int64
void compensate_pressure()
{
    int64_t var1;
    int64_t var2;
    int64_t var3;
    int64_t var4;
    uint32_t pressure;
    uint32_t pressure_min = 3000000;
    uint32_t pressure_max = 11000000;

    var1 = ((int64_t)t_fine) - 128000;
    var2 = var1 * var1 * (int64_t)dig_P6;
    var2 = var2 + ((var1 * (int64_t)dig_P5) * 131072);
    var2 = var2 + (((int64_t)dig_P4) * 34359738368);
    var1 = ((var1 * var1 * (int64_t)dig_P3) / 256) + ((var1 * ((int64_t)dig_P2) * 4096));
    var3 = ((int64_t)1) * 140737488355328;
    var1 = (var3 + var1) * ((int64_t)dig_P1) / 8589934592;

    /* To avoid divide by zero exception */
    if (var1 != 0)
    {
        var4 = 1048576 - adc_p;    // not pressure;  // adc_p?
        var4 = (((var4 * INT64_C(2147483648)) - var2) * 3125) / var1;
        var1 = (((int64_t)dig_P9) * (var4 / 8192) * (var4 / 8192)) / 33554432;
        var2 = (((int64_t)dig_P8) * var4) / 524288;
        var4 = ((var4 + var1 + var2) / 256) + (((int64_t)dig_P7) * 16);
        pressure = (uint32_t)(((var4 / 2) * 100) / 128);

        if (pressure < pressure_min) { pressure = pressure_min; }
        else 
        {
           if (pressure > pressure_max) { pressure = pressure_max; }
        }
    }
    else  { pressure = pressure_min; }

    sensor.pressure = pressure/100;   // hex
//    Pressure = pressure/100;

// pressure_altitude(Pa, P0) calculates Pressure Altitude and creates a string with Meters and Feet
//    pressure_altitude(821.00, 1024.4);  // (Pa, P0) for testing  altitude should be 5997 feet
    pressure_altitude(sensor.pressure/100, pref_work.pref_float/10);
}
//============================================
void compensate_temperature()
{
    int32_t var1;
    int32_t var2;
    int32_t temperature;

    temperature = adc_t;

    var1 = (int32_t)((temperature / 8) - ((int32_t)dig_T1 * 2));

    var1 = (var1 * ((int32_t)dig_T2)) / 2048;
testvars[0] = var1;

    var2 = (int32_t)((temperature / 16) - ((int32_t)dig_T1));
 
    var2 = (((var2 * var2) / 4096) * ((int32_t)dig_T3)) / 16384;
testvars[1] = var2;

    t_fine = var1 + var2;
testvars[2] = t_fine;

    temperature = (t_fine * 5 + 128) / 256;   // Example: T = 2508 = 25.08 deg C
    sensor.temperature_C = temperature/10;    // Example: T = 250 = 25.0 deg C
testvars[3] = temperature;

//   sensor.temperature_C = -300; // for testing
   sensor.temperature_F = ((sensor.temperature_C*18) + 3200)/10;
testvars[4] = sensor.temperature_F;

}
//==============================
static uint32_t compensate_humidity()
{
    int32_t var1;
    int32_t var2;
    int32_t var3;
    int32_t var4;
    int32_t var5;

// Humidity Compensation
// Bosch code
// Returns humidity in %RH as unsigned 32 bit integer in Q22.10 format (22 integer and 10 fractional bits).

// Bosch Code from https://github.com/BoschSensortec/BME280_driver
    var1 = t_fine - (int32_t)76800;
    var2 = (int32_t)adc_h * 16384;           // adc_h is the uncompensated humidity
    var3 = (int32_t)dig_H4 * 1048576;
    var4 = (int32_t)dig_H5 * var1;
    var5 = ( ((var2 - var3) - var4) + (int32_t)16384 ) / 32768;
    var2 = (var1 * (int32_t)dig_H6 ) / 1024;
    var3 = ( (var1 * (int32_t)dig_H3) ) / 2048;
    var4 = ( (var2 * (var3 + (int32_t)32768)) / 1024) + (int32_t)2097152;
    var2 = ( (var4 * (int32_t)dig_H2 ) + 8192) / 16384;
    var3 = var5 * var2;
    var4 = ((var3 / 32768) * (var3 / 32768)) / 128;
    var5 = var3 - ((var4 * (int32_t)dig_H1 ) / 16);
    var5 = (var5 < 0 ? 0 : var5);
    var5 = (var5 > 419430400 ? 419430400 : var5);
    sensor.humidity = ( (uint32_t)var5 / 4096);

    // mine
    sensor.humidity = sensor.humidity/1000;

    if (sensor.humidity > 99)  { sensor.humidity = 99; }
//    sensor.humidity = Humidity;   // hex
   return sensor.humidity;
}
//=========================================
void LED_break()
{
   while(1)  
   {
      gpio_put(LED1,1);
      sleep_ms(250); 
      gpio_put(LED1,0);
      sleep_ms(250); 
   }
}
//=====================================================
// log_wx_data_entry(&flash_entry)
void log_wx_data_entry(struct log *ptr)
{
   log_time();   // time stamp in line

   strcat(line,"Temperature: ");
   temperature_itoa_str(ptr->Ftemperature, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," Degrees    ");

   strcat(line,"Pressure: ");
   pressure_ltoa_str(ptr->Pressure,nstr);        // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb    ");

   strcat(line,"Humidity: ");
   humidity_btoa_str(ptr->Humidity, nstr);
   strcat(line,nstr);

   strcat(line," %\r\n\n");

   uart_puts(UART_ID,line);  
}
//=======================================================
// Convert Binary signed temperature to BCD; 4 digits
// temperature from bme280 is in Celsius 
void temperature_itoa_str(int32_t DegF, char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3;
   unsigned char char1, char2, char3, char4;

   int i;
   unsigned char hcarry;
   unsigned int bin, sign;

// 0x08DD = 2269.; 2269/10 = 226; 226 * 18 = 4084; 4084 + 3200 = 7284 = 72.8 deg F;  
// 0xF723 = -2269.; -2269/10 = -226; 226 * 18 = -4084; -4084 + 3200 = -884 = -8.8 deg F;  

   if(DegF < 0)
   {
      bin = -DegF;
      sign = 1; 
   }
   else
   { 
       bin = DegF;
       sign = 0;
   }

// Convert to BCD
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string; remove leading zeros
    if(sign)    // number is negative
    { 
       str[0] = '-';

       if(char1 != 0x30)
       { 
          str[1] = char1;
          str[2] = char2;
          str[3] = char3;
          str[4] = '.';
          str[5] = char4;
          str[6] = 0;
          for(i=0; i<7; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[1] = char3;
      str[2] = '.';
      str[3] = char4;
      str[4] = 0;
      for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}   // for displaying on lcd later
    }  // end of sign is negative
    else   // sign is positive
    {
       if(char1 != 0x30)
       { 
          str[0] = char1;
          str[1] = char2;
          str[2] = char3;
          str[3] = '.';
          str[4] = char4;
          str[5] = 0;
          for(i=0; i<6; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }
     
       if(char2 != 0x30)
       { 
          str[0] = char2;
          str[1] = char3;
          str[2] = '.';
          str[3] = char4;
          str[4] = 0;
          for(i=0; i<5; i++)  {temperature_lcd[i] = str[i];}  // for displaying on lcd later
          return;
       }  

      // display at least two characters
      str[0] = char3;
      str[1] = '.';
      str[2] = char4;
      str[3] = 0;
      for(i=0; i<4; i++)  {temperature_lcd[i] = str[i];}    // for displaying on lcd later
    }  // end of sign is positive
}
//==========================================================
// Convert hex pressure to BCD; 6 digits
void pressure_ltoa_str(uint32_t bin, char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1, bcd2, bcd3, bcd4, bcd5;
   unsigned char char1, char2, char3, char4, char5;

   int i;
   unsigned char hcarry;

   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;
   bcd4 = 0;
   bcd5 = 0;

   for(i=0; i<24; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x800000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
         bcd2 = bcd2 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD3
      bcd3 = bcd3<<1;
      bcd3 = bcd3 + hcarry;

      if(bcd3 >9)
      {
          bcd3 = bcd3 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD4
      bcd4 = bcd4<<1;
      bcd4 = bcd4 + hcarry;

      if(bcd4 >9)
      {
         bcd4 = bcd4 - 10;
         hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD5
      bcd5 = bcd5<<1;
      bcd5 = bcd5 + hcarry;

      if(bcd5 >9)
      {
         bcd5 = bcd5 - 10;
//         hcarry = 1;
      }
//      else {hcarry = 0;}

   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd5 + 0x30;
   char2 = bcd4 + 0x30;
   char3 = bcd3 + 0x30;
   char4 = bcd2 + 0x30;
   char5 = bcd1 + 0x30;

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = '.';
      str[5] = char5;
      str[6] = 0;      // NULL
      for(i=0; i<7; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }
   
   if(char2 != 0x30)
   { 
      str[0] = char2;
      str[1] = char3;
      str[2] = char4;
      str[3] = '.';
      str[4] = char5;
      str[5] = 0;    // NULL
      for(i=0; i<6; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   if(char3 != 0x30)
   { 
      str[0] = char3;
      str[1] = char4;
      str[2] = '.';
      str[3] = char5;
      str[4] = 0;     // NULL
      for(i=0; i<5; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd
      return;
   }

   str[0] = char4;
   str[1] = '.';
   str[2] = char5;
   str[3] = 0;     // NULL
   for(i=0; i<4; i++)  {pressure_lcd[i] = str[i];}  // for displaying on lcd

   asm(" nop");    // for emulator
}
//=============================================
// Convert Binary signed humidity to BCD - 4 digits
void humidity_btoa_str(uint8_t bin, char *str)      // remove leading zeros; end with NULL
{
   unsigned char bcd0, bcd1;
   unsigned char char1, char2;

   int i;
   unsigned char hcarry;

   if(bin>99)  bin=99;   // maximum number

   bcd0 = 0;
   bcd1 = 0;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
     else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
         bcd1 = bcd1 - 10;
//         hcarry = 1;
      }
       else {hcarry = 0;}
   }  // end of converting to bcd

// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;

// Make the string; remove leading zeros
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = 0;
      for(i=0; i<3; i++)  {humidity_lcd[i] = str[i];}
   }
   else
   {
      str[0] = char2;
      str[1] = 0;
      for(i=0; i<2; i++)  {humidity_lcd[i] = str[i];}  // for displaying on lcd
   }  
}
//==================================================
// make_wx_entry(&flash_entry, flag);   // flag = 1 means get local time  
void make_wx_entry(struct log *ptr)
{
//   uint16_t n;
   uint8_t weekday;

   get_local();

   ptr->ts_number = ts_number;
   ts_number++;

//   weekday = do_day_of_week(9,28,24);   //sat
//   test_sint16(31, weekday);

   weekday = do_day_of_week( (time.local_hex_month), (time.local_hex_date), (time.local_hex_year - 2000) ); 
   ptr->ts_day = time.utc_dow = weekday;
//   test_sint16(32, weekday);

//   ptr->ts_day = time.utc_dow = do_day_of_week( (time.local_hex_month), (time.local_hex_date), (time.local_hex_year - 2000) );  
   // only last 2 digits of the year

//   uint8_bcd_str2(time.local_hex_date,nstr);
   do_clock8(time.local_hex_date);
   ptr->ts_date = ((nstr[0] - 0x30)<<4) + (nstr[1] - 0x30);

//   uint8_bcd_str2(time.local_hex_month,nstr);
   do_clock8(time.local_hex_month);
   ptr->ts_month = ((nstr[0] - 0x30)<<4) + (nstr[1] - 0x30);

   uint8_bcd_str2(time.local_hex_year-2000,nstr);
   ptr->ts_year = ((nstr[0] - 0x30)<<4) + (nstr[1] - 0x30);

//   uint8_bcd_str2(time.local_hex_hours,nstr);
   do_clock8(time.local_hex_hours);
   ptr->ts_hours = ((nstr[0] - 0x30)<<4) + (nstr[1] - 0x30);

//   uint8_bcd_str2(time.local_hex_mins,nstr);
   do_clock8(time.local_hex_mins);
   ptr->ts_minutes = ((nstr[0] - 0x30)<<4) + (nstr[1] - 0x30);

//   uint8_bcd_str2(time.local_hex_secs,nstr);
   do_clock8(time.local_hex_secs);
   ptr->ts_seconds = ((nstr[0] - 0x30)<<4) + (nstr[1] - 0x30);

   ptr->Ftemperature = sensor.temperature_F;
   
   ptr->Pressure = sensor.pressure;

   ptr->Humidity = sensor.humidity;

   strcpy(ptr->pressure_altitude_str, pressure_altitude_str);

   strcpy(ptr->sats, sats_in_view);
   strcpy(ptr->gps_altitude_str, gps_altitude_str);
   strcpy(ptr->gps_latitude_str, gps_latitude_str);
   strcpy(ptr->gps_longitude_str, gps_longitude_str);

   ptr->flag = 0xA5;    // flag for non-blank

   strcpy(ptr->pref_str, pref_work.pref_str);  // Barometer reference in inches Hg and mb 
}
//==================================================
// make_wx_entry(&flash_entry);  utc
void make_wx_entry_utc(struct log *ptr)
{
   ptr->ts_number = ts_number;
   ts_number++;

   ptr->ts_date = (uint8_t)time.utc_bcd_date;
   ptr->ts_day = (uint8_t)time.utc_dow;
   ptr->ts_month = (uint8_t)time.utc_bcd_month;
   ptr->ts_year = (uint8_t)(time.utc_bcd_year - 0x2000);  // 0x2000 because it is bcd

   ptr->ts_hours = (uint8_t)time.utc_bcd_hours;
   ptr->ts_minutes = (uint8_t)time.utc_bcd_mins;
   ptr->ts_seconds = (uint8_t)time.utc_bcd_secs;

   ptr->Ftemperature = sensor.temperature_F;
   
   ptr->Pressure = sensor.pressure;

   ptr->Humidity = sensor.humidity;

   strcpy(ptr->pressure_altitude_str, pressure_altitude_str);

   strcpy(ptr->sats, sats_in_view);
   strcpy(ptr->gps_altitude_str, gps_altitude_str);
   strcpy(ptr->gps_latitude_str, gps_latitude_str);
   strcpy(ptr->gps_longitude_str, gps_longitude_str);

   ptr->flag = 0xA5;    // flag for non-blank

   strcpy(ptr->pref_str, pref_work.pref_str);  // Barometer reference in inches Hg and mb 
}
//================================
// show_entry(&flash_entry);
void show_entry(struct log *ptr)
{
   uint16_t m, n;

// ts_number is the log number starting from Reset
   my_itoa(ptr->ts_number,nstr);  // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

// day name
   n = (ptr->ts_day)- 1;   // use it as an index
   for(m=0; m<3; m++)  { nstr[m] = day_array[n][m]; }
   nstr[m] = 0;
   strcat(line,nstr);
   strcat(line,"    ");   

   line_fill(ptr->ts_month, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_date, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_year, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(ptr->ts_hours, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_minutes, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_seconds, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   strcat(line, "Temperature: ");
   temperature_itoa_str( ptr->Ftemperature, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," F     ");
   
   strcat(line, "Pressure: ");
   pressure_ltoa_str(ptr->Pressure, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb      ");

   strcat(line, "Humidity: ");
   humidity_btoa_str( ptr->Humidity, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," %\n");
   uart_puts(UART_ID, line); 

   strcpy(line, "Barometric Pressure:  ");
   strcat(line, ptr->pref_str);
   strcat(line," \n");
   uart_puts(UART_ID, line); 

   strcpy(line, "Latitude: ");
   strcat(line, ptr->gps_latitude_str);
   strcat(line, "      ");

   strcat(line, "Longitude: ");
   strcat(line, ptr->gps_longitude_str);
   strcat(line, "\n");
   uart_puts(UART_ID, line); 

   strcpy(line, "GPS Altitude: ");
   strcat(line, ptr->gps_altitude_str);
   strcat(line, "       ");
   
   strcat(line, "Pressure Altitude: ");
   strcat(line, ptr->pressure_altitude_str);
   strcat(line," \n");
   uart_puts(UART_ID, line); 
}
//========================================
// show_time();  // only time in flash_entry
void show_time()
{
   struct log *ptr = &flash_entry;
   uint16_t m, n;

   make_wx_entry(&flash_entry);  // make a log entry from time, sensor.temperature_F, p, Humidity 

   strcpy(line,"ZDA (UTC)   ");

// day name
   n = (ptr->ts_day)- 1;   // use it as an index
   for(m=0; m<3; m++)  { nstr[m] = day_array[n][m]; }
   nstr[m] = 0;
   strcat(line,nstr);
   strcat(line,"    ");   

   line_fill(ptr->ts_month, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_date, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_year, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(ptr->ts_hours, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_minutes, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_seconds, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"\n\n");

   uart_puts(UART_ID, line); 
}
//========================================
//&&&&&&&&&&&&&&&&&
/*!
 * @brief This internal API is used to compensate the raw humidity data and
 * return the compensated humidity data in double data type.
 */
static double dcompensate_humidity()
{
    double humidity;
    double humidity_min = 0.0;
    double humidity_max = 100.0;
    double var1;
    double var2;
    double var3;
    double var4;
    double var5;
    double var6;

    var1 = ( (double)t_fine ) - 76800.0;
    var2 = (((double)dig_H4) * 64.0 + (((double)dig_H5) / 16384.0) * var1);
    var3 = adc_h - var2;   // adc_h is the uncompensated humidity
    var4 = ((double)dig_H2) / 65536.0;
    var5 = (1.0 + (((double)dig_H3) / 67108864.0) * var1);
    var6 = 1.0 + (((double)dig_H6) / 67108864.0) * var1 * var5;
    var6 = var3 * var4 * (var5 * var6);
    humidity = var6 * (1.0 - ((double)dig_H1) * var6 / 524288.0);

    if (humidity > humidity_max)
    {
        humidity = humidity_max;
    }
    else if (humidity < humidity_min)
    {
        humidity = humidity_min;
    }

    return humidity;
}
//========================================
/*!
 * @brief This internal API is used to compensate the raw pressure data and
 * return the compensated pressure data in double data type.
 */
static double dcompensate_pressure()
{
    double var1;
    double var2;
    double var3;
    double pressure;
    double pressure_min = 30000.0;
    double pressure_max = 110000.0;

    var1 = ((double)t_fine / 2.0) - 64000.0;
    var2 = var1 * var1 * ((double)dig_P6) / 32768.0;
    var2 = var2 + var1 * ((double)dig_P5) * 2.0;
    var2 = (var2 / 4.0) + (((double)dig_P4) * 65536.0);
    var3 = ((double)dig_P3) * var1 * var1 / 524288.0;
    var1 = (var3 + ((double)dig_P2) * var1) / 524288.0;
    var1 = (1.0 + var1 / 32768.0) * ((double)dig_P1);

    /* Avoid exception caused by division by zero */
    if (var1 > (0.0))
    {
        pressure = 1048576.0 - (double)adc_p;   // adc_p is uncompensated pressure
        pressure = (pressure - (var2 / 4096.0)) * 6250.0 / var1;
        var1 = ((double)dig_P9) * pressure * pressure / 2147483648.0;
        var2 = pressure * ((double)dig_P8) / 32768.0;
        pressure = pressure + (var1 + var2 + ((double)dig_P7)) / 16.0;

        if (pressure < pressure_min)
        {
            pressure = pressure_min;
        }
        else if (pressure > pressure_max)
        {
            pressure = pressure_max;
        }
    }
    else /* Invalid case */
    {
        pressure = pressure_min;
    }

    return pressure;
}
//================================
/*!
 * @brief This internal API is used to compensate the raw temperature data and
 * return the compensated temperature data in double data type.
 */
static double dcompensate_temperature()
{
    double var1;
    double var2;
    double temperature;
    double temperature_min = -40;
    double temperature_max = 85;
    uint16_t sign;

// adc_t is uncompensated temperature
    var1 = (((double)adc_t) / 16384.0 - ((double)dig_T1) / 1024.0);
    var1 = var1 * ((double)dig_T2);
    var2 = (((double)adc_t) / 131072.0 - ((double)dig_T1) / 8192.0);
    var2 = (var2 * var2) * ((double)dig_T3);
    t_fine = (int32_t)(var1 + var2);
    temperature = (var1 + var2) / 5120.0;

    if (temperature < temperature_min)
    {
        temperature = temperature_min;
    }
    else if (temperature > temperature_max)
    {
        temperature = temperature_max;
    }

// Scale and convert to Fahrenheit
    if(temperature<0)  {sign = 1;}
    else     {sign = 0;}

//   T = temperature/10;   // Example: 2508 -> 250 

    if(sign)  { temperature = (temperature * 18.) - 320.;}
    else { temperature = (temperature * 18.) + 320.;}    

//    temperature = lbin/10;  

    return temperature;
}
//===============================================
#define BME280_CONCAT_BYTES(msb, lsb)    (((uint16_t)msb << 8) | (uint16_t)lsb)

static void test_bosch_parse_calib_data()
{
    int16_t dig_H4_msb;
    int16_t dig_H4_lsb;
    int16_t dig_H5_msb;
    int16_t dig_H5_lsb;

    dig_T1 = BME280_CONCAT_BYTES(caldata[1], caldata[0]);
    dig_T2 = (int16_t)BME280_CONCAT_BYTES(caldata[3], caldata[2]);
    dig_T3 = (int16_t)BME280_CONCAT_BYTES(caldata[5], caldata[4]);

    dig_P1 = BME280_CONCAT_BYTES(caldata[7], caldata[6]);
    dig_P2 = (int16_t)BME280_CONCAT_BYTES(caldata[9], caldata[8]);
    dig_P3 = (int16_t)BME280_CONCAT_BYTES(caldata[11], caldata[10]);
    dig_P4 = (int16_t)BME280_CONCAT_BYTES(caldata[13], caldata[12]);
    dig_P5 = (int16_t)BME280_CONCAT_BYTES(caldata[15], caldata[14]);
    dig_P6 = (int16_t)BME280_CONCAT_BYTES(caldata[17], caldata[16]);
    dig_P7 = (int16_t)BME280_CONCAT_BYTES(caldata[19], caldata[18]);
    dig_P8 = (int16_t)BME280_CONCAT_BYTES(caldata[21], caldata[20]);
    dig_P9 = (int16_t)BME280_CONCAT_BYTES(caldata[23], caldata[22]);

    dig_H1 = humdata[0];  //  7:0

    dig_H2 = (int16_t)BME280_CONCAT_BYTES(humdata[2], humdata[1]);
    dig_H3 = humdata[3];
    dig_H4_msb = (int16_t)(int8_t)humdata[4] * 16;
    dig_H4_lsb = (int16_t)(humdata[5] & 0x0F);
    dig_H4 = dig_H4_msb | dig_H4_lsb;
    dig_H5_msb = (int16_t)(int8_t)humdata[6] * 16;
    dig_H5_lsb = (int16_t)(humdata[5] >> 4);
    dig_H5 = dig_H5_msb | dig_H5_lsb;
    dig_H6 = (int8_t)humdata[7];
}
//===============================================
// unsigned char to ASCII string, 3 digits
void uint8_bcd_str(unsigned char bin, char *str)    
{
// First Convert unsigned hex byte to BCD 
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
 
   unsigned char char1, char2, char3;
   unsigned char hcarry;

//bin = 0xA2;
   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }
//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}
//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
          bcd2 = bcd2 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}
   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Put it in the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = 0;      // NULL
}
//=================================================
// unsigned char to ASCII string, 2 digits
void uint8_bcd_str2(unsigned char bin, char *str)    
{
// First Convert unsigned hex byte to BCD 
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
 
   unsigned char char1, char2;
   unsigned char hcarry;

//bin = 0xA2;
   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }
//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
//      else {hcarry = 0;}
   }

// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;

// Put it in the string
   str[0] = char1;
   str[1] = char2;
   str[2] = 0;      // NULL
}
//=================================================
// signed char to ASCII string, 3 digits
void sint8_bcd_str(int8_t bin, char *str)    
{
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
 
   unsigned char char1, char2, char3, sflag;
   unsigned char hcarry;

   if(bin <0)
   {
      bin = -bin;
      sflag = 1;
   }
   else {sflag = 0;}

// Convert unsigned hex byte to BCD 
   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }
//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}
//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
          bcd2 = bcd2 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Put it in the string
   if(sflag)
   {
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = 0;      // NULL

   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = 0;      // NULL
   }
}
//=================================================
// uint16 to ASCII string, 5 digits
void uint16_bcd_str(uint16_t bin, char *str)
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char char1, char2, char3, char4, char5;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }
//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}
//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}
//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
       hcarry = 1;
     }
         else {hcarry = 0;}
//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
//       hcarry = 1;
     }
//         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd4 + 0x30;
   char2 = bcd3 + 0x30;
   char3 = bcd2 + 0x30;
   char4 = bcd1 + 0x30;
   char5 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = char5;
   str[5] = 0;  // NULL
}
//=================================================
// uint16 to ASCII string to uart - 4 digits
void uint16_bcd_str4(uint16_t bin, char *str)   //unsigned int to BCD string
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
       hcarry = 1;
     }
//         else {hcarry = 0;}
   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = 0;  // NULL
}
//=================================================
// sint16 to ASCII string, 5 digits
void sint16_bcd_str(int16_t bin, char *str)
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char char1, char2, char3, char4, char5, sflag;
   int i;
   unsigned char hcarry;

   if(bin <0)
   {
      bin = -bin;
      sflag = 1; 
   }
   else {sflag = 0; }

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }
//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}
//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}
//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
       hcarry = 1;
     }
         else {hcarry = 0;}
//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
//       hcarry = 1;
     }
//         else {hcarry = 0;}
   }

// Convert to ascii
   char1 = bcd4 + 0x30;
   char2 = bcd3 + 0x30;
   char3 = bcd2 + 0x30;
   char4 = bcd1 + 0x30;
   char5 = bcd0 + 0x30;

// Make the string
   if(sflag)
   {
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = char4;
      str[5] = char5;
      str[6] = 0;      // NULL
   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = char5;
      str[5] = 0;      // NULL
   }
}
//===============================================
// uint32 to ASCII string, 10 digits 
void uint32_bcd_str(uint32_t bin, char *str)
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
   unsigned char bcd7 = 0;
   unsigned char bcd8 = 0;
   unsigned char bcd9 = 0;
   unsigned char char1, char2, char3, char4, char5, char6, char7, char8, char9, char10;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<32; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }
//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}
//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
         bcd5 = bcd5 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
         bcd6 = bcd6 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD7
     bcd7 = bcd7<<1;
     bcd7 = bcd7 + hcarry;

     if(bcd7 >9)
     {
         bcd7 = bcd7 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD8
     bcd8 = bcd8<<1;
     bcd8 = bcd8 + hcarry;

     if(bcd8 >9)
     {
         bcd8 = bcd8 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD9
     bcd9 = bcd9<<1;
     bcd9 = bcd9 + hcarry;

     if(bcd9 >9)
     {
         bcd9 = bcd9 - 10;
//         hcarry = 1;
     }
//   else {hcarry = 0;}
   }  // end for

// Convert to ascii
   char1 = bcd9 + 0x30;
   char2 = bcd8 + 0x30;
   char3 = bcd7 + 0x30;
   char4 = bcd6 + 0x30;
   char5 = bcd5 + 0x30;
   char6 = bcd4 + 0x30;
   char7 = bcd3 + 0x30;
   char8 = bcd2 + 0x30;
   char9 = bcd1 + 0x30;
   char10 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = char5;
   str[5] = char6;
   str[6] = char7;
   str[7] = char8;
   str[8] = char9;
   str[9] = char10;
   str[10] = 0;  // NULL
}
//===============================================
// sint32 to ASCII string, 10 digits
void sint32_bcd_str(int32_t bin, char *str)    // int32_t to BCD string
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
   unsigned char bcd7 = 0;
   unsigned char bcd8 = 0;
   unsigned char bcd9 = 0;
   unsigned char char1, char2, char3, char4, char5, char6, char7, char8, char9, char10, sflag;
   int i;
   unsigned char hcarry;

   if(bin <0)
   {
      bin = -bin;
      sflag = 1;
   }
   else { sflag = 0; }

// Convert Binary to BCD
   for(i=0; i<32; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }
//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}
//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
         bcd5 = bcd5 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
         bcd6 = bcd6 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD7
     bcd7 = bcd7<<1;
     bcd7 = bcd7 + hcarry;

     if(bcd7 >9)
     {
         bcd7 = bcd7 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD8
     bcd8 = bcd8<<1;
     bcd8 = bcd8 + hcarry;

     if(bcd8 >9)
     {
         bcd8 = bcd8 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}
//---------- BCD9
     bcd9 = bcd9<<1;
     bcd9 = bcd9 + hcarry;

     if(bcd9 >9)
     {
         bcd9 = bcd9 - 10;
//         hcarry = 1;
     }
//   else {hcarry = 0;}
   }  // end for

// Convert to ascii
   char1 = bcd9 + 0x30;
   char2 = bcd8 + 0x30;
   char3 = bcd7 + 0x30;
   char4 = bcd6 + 0x30;
   char5 = bcd5 + 0x30;
   char6 = bcd4 + 0x30;
   char7 = bcd3 + 0x30;
   char8 = bcd2 + 0x30;
   char9 = bcd1 + 0x30;
   char10 = bcd0 + 0x30;

// Make the string
   if(sflag)
   {
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = char4;
      str[5] = char5;
      str[6] = char6;
      str[7] = char7;
      str[8] = char8;
      str[9] = char9;
      str[10] = char10;
      str[11] = 0;  // NULL
   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = char5;
      str[5] = char6;
      str[6] = char7;
      str[7] = char8;
      str[8] = char9;
      str[9] = char10;
      str[10] = 0;  // NULL
   }
}
//===============================================
// unsigned 8 bits to ASCII string as 2 hex digits
void uint8_hex_str(uint8_t bin, char *str)   //unsigned char to hex string    
{
   unsigned char1, char2, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}
    
// Make string
   str[0] = char1;
   str[1] = char2;
   str[2] = 0;
}
//==========================================================
// uint16 to ASCII string as 4 hex digits
void uint16_hex_str(uint16_t bin, char *str)
{
   unsigned char1, char2, char3, char4, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char4 = temp + 0x30;}
   else
     {char4 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char3 = temp + 0x30;}
   else
     {char3 = temp + 0x37;}

   temp = bin>>8 & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>12 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}

   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = 0;  // NULL
}
//============================================
// uint32 to ASCII string as 8 hex digits
void uint32_hex_str(unsigned long bin, char *str)  // uint32 to ASCII string as 8 hex digits
// long int hex to string, space between ints
{
// Convert Binary to BCD
   unsigned int m, n;
   char temp[10];

   m = 7;
   for(n=0; n<8; n++)
   {
      temp[m] = bin & 0x0F;
      m--;
      bin = bin>>4;
   }
   asm(" nop");    // for emulator

   // Convert to ascii
   for(n=0; n<8; n++)
   {
     if(temp[n]<10)  { temp[n] = temp[n] + 0x30; }
     else            { temp[n] = temp[n] + 0x37; }
   }
  
// Format the string
   for(n=0; n<4; n++)  { str[n] = temp[n]; }
   str[4] = ' ';

   for(n=4; n<8; n++)  { str[n+1] = temp[n]; }
   str[9] = 0x00;  // NULL   Now it is a string

   asm(" nop");    // for emulator
}
//========================================================
void bme280_parameters()
{
//   uint16_t n;
   uint32_t result;

/*
   log_size = sizeof(struct log);
   uint16_bcd_str(log_size, nstr);
   strcpy(line,"\n\nSize of struct log: ");
   strcat(line,nstr);
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);   // send it out
*/

   strcpy(line,"Device ID: ");  // Get device ID
   itoa_hex(device_id);
   strcat(line,nstr);
   if(device_id == 0x60)  { strcat(line, "  BME-280\n\n"); }
   if(device_id == 0x58)  { strcat(line, "  BMP-280\n\n"); }
   if( (device_id != 0x60) && (device_id != 0x58) )  { strcat(line, "unknown device\n\n"); }
   uart_puts(UART_ID, line);   // send it out

   sleep_ms(3000);
   
   getcal();     // Do calibration data for temperature and humidity and make digs
   log_cal();
   uart_puts(UART_ID, "My Digs\n");
   log_digs();

   read_bme280();
   compensate_temperature();  // sensor.temperature_F = temperature in Fahrenheit
   log_vars();

   compensate_pressure();     // sensor.pressure = pressure in mb
   compensate_humidity();     // sensor.humidity in %

   show_temperature = sensor.temperature_F;   // for testing the sensor  7403 is 2430 is 24.3 degrees C is 75.7 degrees F
   show_humidity = sensor.humidity;  // for testing the sensor  23 is 23%
   show_pressure = sensor.pressure;  // for testing the sensor  "96386" equals 96386 Pa = 963.86 hPa

// Did I get the same Digs as Bosch?
   test_bosch_parse_calib_data();
   uart_puts(UART_ID, "Bosch Digs\n");
   log_digs();

// How does 32-bit integer compare to floating point?
   uart_puts(UART_ID, "For Comparing 32-bit integer to floating point\n");

   result = dcompensate_temperature() * 10;  // sensor.temperature_F = temperature in Fahrenheit
   strcpy(line,"Temperature:  "); 
   sint32_bcd_str(result, nstr);             // int32_t to ASCII string, 8 digits
   strcat(line, nstr);
   strcat(line,"\n");
   uart_puts(UART_ID, line);

   result = dcompensate_pressure();    // global p = pressure in mb
   strcpy(line,"Pressure:  "); 
   uint32_bcd_str(result, nstr);       // uint32_t to ASCII string, 8 digits
   strcat(line, nstr);
   strcat(line,"\n");
   uart_puts(UART_ID, line);

   result = dcompensate_humidity() * 10;     // global Humidity in %
   strcpy(line,"Humidity:  "); 
   uint32_bcd_str(result, nstr);             // uint32_t to ASCII string, 8 digits  
   strcat(line, nstr);
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);

   asm(" nop");               // for debugger, break here to check that the sensor is working

   log_raw();

   uart_puts(UART_ID, "32-bit integer data\n");
   log_wx_data_ts();    // send data out through uart for logging the wx data
   show_wx_data_lcd();

   sleep_ms(2000);
}
//=============================================
/*
Altitude from air pressure

A good discussion is at:  
https://physics.stackexchange.com/questions/333475/how-to-calculate-altitude-from-current-temperature-and-pressure

I am using an approximation of the Barometric Formula (https://en.wikipedia.org/wiki/Barometric_formula) and is 
a good approximation for altitudes below 9000 meters (about 30,000 feet) and when the temperature at sea level is 
15 deg C (59 degrees F). The U.S. Weather Service does not report the temperature at sea level when they report the 
barometric pressure so it must be the standard 15 deg C. Besides, unless you live on a coast where would you measure 
the temperature?

The equation assumes a standard lapse rate. The lapse rate is more complicated than you might think: 
https://en.wikipedia.org/wiki/Lapse_rate and global warming is affecting the lapse rate.

     H = 44330 * [1 - Pa/P0 ^ (0.190295)]

     Pa = air pressure at altitude							
     P0 = air pressure at sea level (barometer)							
     H = pressure altitude in meters							

Height Above Mean Sea Level (AMSL) is not an exact science. However, it is very useful when comparing altitudes. That is
why airports report their altitude (however it is they determine what it is) and the current barometric pressure at the 
airport. As long as all of the aircraft pilots in the vicinity use the same values and the same methods they know where 
they are vertically in relation to everyone else.

The Barometric Altimeter was invented in the 1920s by Paul Kollsman. https://appel.nasa.gov/2010/02/27/ao_2-1_f_kollsman-html/
Kollsman was one of the great inventors of the 20th Century. I don't think he gets the recognition that he deserves.

How did Kollsman's altimeter work? It was purely mechanical. 
See https://www.aopa.org/news-and-media/all-news/2019/july/flight-training-magazine/how-it-works-altimeter

And you may find this video useful: https://www.youtube.com/watch?v=glcgxoGLHrQ

GPS altitude is less accurate than the Pressure Altitude, sometimes much less accurate. This project allows you to compare them.
*/

void pressure_altitude(float Pa, float P0)     // Takes 46 us
{
//   Pa = 821.00;  // for testing
//   P0 = 1024.4;  // for testing

   float temp;
   float K = 0.190295;
   float result;
   int32_t palt_m, palt_f;  // meters, feet

   temp = Pa/P0;
   result = powf(temp, K);
   result = 1.0 - result;
   result = result * 44330;

   palt_m = result;
   uint16_bcd_str(palt_m, nstr);   // unsigned int32 to hex string to uart   
   strcpy(line, nstr);
   strcat(line, " M   ");

   palt_f = result * 3.2808399;
   uint16_bcd_str(palt_f, nstr);   // unsigned int32 to hex string to uart   
   strcat(line, nstr);
   strcat(line, " F\n\n");
   strcpy(pressure_altitude_str, line);   // global
}
//===================================
void lcd_altitude_feet()
{
   uint16_t n;
   unsigned char data;

   SetCmdMode();        // set for commands
   lcd_byte(0x01);      // clear lcd, both lines

   domsg(&amsg1);

// GPS Altitude (Feet)
    lcd_wr_instr(0xC0);   // position
    for(n=12; n<17; n++)
    {
        data = gps_altitude_str[n];
        if( (n==12) && (data == 0x30) ) { data = ' ';}
        lcd_wr_data(data);
    }

// Pressure Altitude (Feet)
    lcd_wr_instr(0xC8);   // position
    for(n=10; n<15; n++)
    {
        data = pressure_altitude_str[n];
        if( (n==10) && (data == 0x30) ) { data = ' ';}    
        lcd_wr_data(data);
    }
}
//======================================
void start_bme280()
{
   i2c_outbuffer[0] = 0xD0;  // register address
   i2c_inbuffer[0] = 0x00;   

   // Display the BME device id
   i2c_outbuffer[0] = 0xD0;  // register address
   i2c_write_blocking(i2c_default, BME_ID, i2c_outbuffer, 1, true);
   i2c_read_blocking(i2c_default, BME_ID, i2c_inbuffer, 1, false);
   device_id = i2c_inbuffer[0];
      
   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen

   domsg(&bmsg1);   // "Device ID"
   lcd_wr_hex(0x8A,device_id);
   device_id = i2c_inbuffer[0];

   if(device_id == 0x60)   { domsg(&bmsg2); }  // "BME-280"
   if(device_id == 0x58)   { domsg(&bmsg3); }  // "BMP-280"
   if( (device_id != 0x60) && (device_id != 0x58) ) { domsg (&bmsg4); }  // Unknown

   strcpy(line,"\nDevice ID: ");  // Get device ID
   itoa_hex(device_id);
   strcat(line,nstr);
   if(device_id == 0x60)  { strcat(line, "  BME-280\n"); }
   if(device_id == 0x58)  { strcat(line, "  BMP-280\n"); }
   if( (device_id != 0x60) && (device_id != 0x58) )  { strcat(line, "unknown device\n\n"); }
   uart_puts(UART_ID, line);   // send it out
   sleep_ms(2000);

   bme_init();   // Initialize bme
   getcal();     // Do calibration data for temperature and humidity and make digs

   read_bme280();
   compensate_temperature();     // sensor.temperature_F = temperature in Fahrenheit
   compensate_pressure();        // global p = pressure in mb
   compensate_humidity();        // global Humidity in %

//   make_wx_entry(&flash_entry);  // make a log entry from sensor.temperature_F, p, Humidity 
   make_wx_entry_utc(&flash_entry);  // make a log entry from temperature, pressure, and humidity; we don't have local time yet

   show_bme(&flash_entry);
}
//============================================
// show_bme(&flash_entry);
void show_bme(struct log *ptr)
{
//   uint16_t m, n;

   strcpy(line, "Temperature: ");
   temperature_itoa_str( ptr->Ftemperature, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," F     ");
   
   strcat(line, "Pressure: ");
   pressure_ltoa_str(ptr->Pressure, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb      ");

   strcat(line, "Humidity: ");
   humidity_btoa_str( ptr->Humidity, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," %\n\n");
   uart_puts(UART_ID, line); 
}
//========================================
#if NO
void test_week_day()
{
   uint8_t weekday;
   
   weekday = do_day_of_week(9,28,24);   //sat
   test_sint16(30, weekday);

   weekday = do_day_of_week(9,29,24);  // sun
   test_sint16(30, weekday);

   weekday = do_day_of_week(9,30,24);  // mon
   test_sint16(30, weekday);

   weekday = do_day_of_week(10,1,24);  // tue
   test_sint16(30, weekday);

   weekday = do_day_of_week(10,2,24);   //wed
   test_sint16(30, weekday);
 
   weekday = do_day_of_week(10,3,24);   // thu
   test_sint16(30, weekday);

   weekday = do_day_of_week(10,4,24);  // fri
   test_sint16(30, weekday);

   weekday = do_day_of_week(10,5,24);  // sat
   test_sint16(30, weekday);

   weekday = do_day_of_week(10,6,24);  // sun
   test_sint16(30, weekday);
}
#endif
//==========================================================
void show_stuff()    // show some stuff
{
    strcpy(line,"Temperature C: ");
    sint32_bcd_str(sensor.temperature_C, nstr);
    strcat(line, nstr);
    strcat(line,"\n");
    uart_puts(UART_ID,line);
 
    strcpy(line,"Temperature F: ");
    sint32_bcd_str(sensor.temperature_F, nstr);
    strcat(line, nstr);
    strcat(line,"\n");
    uart_puts(UART_ID,line);

    strcpy(line,"Pressure mb: ");
    sint32_bcd_str(sensor.pressure, nstr);
    strcat(line, nstr);
    strcat(line,"\n");
    uart_puts(UART_ID,line);

    strcpy(line,"Humidity %: ");
    sint32_bcd_str(sensor.humidity, nstr);
    strcat(line, nstr);
    strcat(line,"\n\n");
    uart_puts(UART_ID,line);
}
//==============================================
