// For bme-280
#include "common.h"
//----------------------------------------------------------------
const uint16_t structlogsize = 180;   // size of log struct (for logging in Flash)

#define hum_lsb 0xFE;       // humidity byte 0      Read Only  bmpdata[7]
#define hum_msb 0xFD;       // humidity byte 1      Read Only  bmpdata[6]
#define temp_xlsb 0xFC;     // temperature byte 0   Read Only  bmpdata[5]   R,Address,Data
#define temp_lsb 0xFB;      // temperature byte 1   Read Only  bmpdata[4]
#define temp_msb 0xFA;      // temperature byte 2   Read Only  bmpdata[3]
#define press_xlsb 0xF9;    // pressure byte 0      Read Only  bmpdata[2]
#define press_lsb 0xF8;     // pressure byte 1      Read Only  bmpdata[1]
#define press_msb 0xF7;     // pressure byte 2      Read Only  bmpdata[0]

#define bme_config 0xF5;    // configuration        Read/Write  R/W,Address;Data
#define bme_ctrl_meas 0xF4; // control measurement  Read/Write  R/W,Address;Data
#define bme_status 0xF3;    // status               Read Only
#define bme_reset 0xE0;     // reset                Write Only  /W,Address;Data

#define bme_id_reg 0xD0;    // part id = 0x58       Read Only

uint16_t logging;
unsigned char i2c_outbuffer[10];
unsigned char i2c_inbuffer[128];
unsigned char i2c_read_byte(unsigned char);

void log_raw(void);
void bme_init(void);
void do_bme280(void);
void show_wx_data_lcd();
void log_wx_data_ts(void);
void i2c_read(unsigned char, unsigned int);
void i2c_write_byte_bme(unsigned char, unsigned char);
void i2c_write(unsigned char, unsigned int);
uint16_t wait_ready(void);
void log_time(void);   // time stamp in 'line' string
void line_fill(unsigned char, char *, unsigned char, unsigned char);

unsigned char bmedata[8];
unsigned char caldata[24];      // bme calibration data dig T1 -T3, dig P1 - P9, dighum
unsigned char humdata[8];

uint16_t dig_T1;
int16_t dig_T2;
int16_t dig_T3;

uint16_t dig_P1;
int16_t dig_P2;
int16_t dig_P3;
int16_t dig_P4;
int16_t dig_P5;
int16_t dig_P6;
int16_t dig_P7;
int16_t dig_P8;
int16_t dig_P9;

uint8_t dig_H1;
int16_t dig_H2;
uint8_t dig_H3;
int16_t dig_H4;
int16_t dig_H5;
int8_t dig_H6;

int32_t t_fine;
int32_t adc_t, adc_temp, adc_p;
uint32_t adc_h;

struct data sensor;

void pressure_altitude(float, float);

char nstr[256];   // new string segment
char line[512];   // string for uart
unsigned char device_id;
extern unsigned char device_idb;

void itoa_hex(unsigned char);
void itoa_hex_int(unsigned int);
void lcd_byte(unsigned char);    // clear screen
void read_bme280(void);
void getcal(void);
void log_cal(void);

extern void lcd_wr_hex(unsigned int, unsigned char);
extern void lcd_wr_data(unsigned char);
extern void do_led_break(void);
extern void SetCmdMode(void);
extern void lcd_wr_instr(unsigned char);
extern void display_utc_lcd(void);
extern uint16_t zda_flag;
extern void do_local(int8_t, uint16_t, uint16_t);

unsigned char temperature_lcd[10];
unsigned char humidity_lcd[4];
unsigned char pressure_lcd[10];
unsigned char dev[6];

void pressure_ltoa(char *);
void temperature_itoa(char *);
void humidity_btoa(char *);
void devid(char *str);

unsigned char temperature_bcd[5];
unsigned char humidity_bcd[2];
unsigned char pressure_bcd[6];

signed int temperature;
unsigned long pressure;
unsigned char humidity;

char pressure_altitude_str[32];

void log_digs(void);
unsigned char test_buffer[32];
signed long testvars[32];
void log_vars(void);
void lcd_display(void);

unsigned long show_temperature, show_humidity, show_pressure;
//=========================================
void compensate_pressure(void);
void compensate_temperature(void);
static uint32_t compensate_humidity(void);

extern unsigned char time_buffer[];

void log_time(void);
void my_itoa(unsigned int,char *);

extern uint16_t secs_flag;
void LED_break(void);
extern unsigned char uart_char;

unsigned char temp_buffer[256];
extern struct log flash_entry;    
extern const char version[64];
extern uint16_t Page_Index, Last_Page, Sector_Index;
extern void do_start_date(uint16_t);
extern char start_date[];
extern uint16_t ts_number;
extern unsigned int Entry_Read_Index, Flash_Write_Index, erase_flag, reset_rx_start;
void do_uart_RX(void);
extern void make_entry(void);
extern void test_read_sector(uint16_t);
extern void myflash_erase_sector(uint16_t);
extern void erase_sector(uint16_t);
extern void erase_all_sectors(void);
extern void dump_flash(uint32_t, uint32_t);
extern void write_page(uint32_t, unsigned char *);
extern void test10(void);
extern void show_log(struct log *);
void show_entry(struct log *);
void show_time(void);
void show_bme(struct log *);
extern const char day_array[7][3];
void my_hextoa2(unsigned char, char *);

extern void do_B_reference(uint16_t);
extern void show_pref();
extern void erase_bref_flash(void);

// For Saving the Barometric Reference Data
struct pref
{
    uint16_t pref_bcd;
    float pref_float; 
    uint32_t pref_int32;
    char pref_str[32];    // String, example: 29.92" Hg   1013.2 mb
};
extern struct pref pref_work;

void bme280_parameters(void);
void log_wx_data_entry(struct log *);
void temperature_itoa_str(int32_t, char *);      // remove leading zeros; end with NULL
void pressure_ltoa_str(uint32_t, char *);        // remove leading zeros; end with NULL
void humidity_btoa_str(uint8_t, char *);
void make_wx_entry_utc(struct log *);
void make_wx_entry(struct log *);
extern void show_log_page(uint32_t);
uint16_t bme_log_count, bme_log_flag;
//==================
static double dcompensate_humidity(void);
static double dcompensate_pressure(void);
static double dcompensate_temperature(void);

static void test_bosch_parse_calib_data(void);

void uint8_bcd_str(unsigned char, char *);   // unsigned char to ASCII string, 3 digits  
void uint8_bcd_str2(unsigned char, char *);  // unsigned char to ASCII string, 2 digits
extern void do_clock8(uint8_t);              // does it with a lookup table, fast
void sint8_bcd_str(int8_t, char *);          // signed char to ASCII string, 3 digits    
void uint16_bcd_str(uint16_t, char *);       // uint16 to ASCII string, 5 digits
void uint16_bcd_str4(uint16_t, char *);      // uint16 to ASCII string, 4 digits
void sint16_bcd_str(int16_t, char *);        // int16 to ASCII string, 5 digits
void uint32_bcd_str(uint32_t, char *);       // uint32_t to ASCII string, 8 digits
void sint32_bcd_str(int32_t, char *);        // int32_t to ASCII string, 8 digits
void uint8_hex_str(uint8_t, char *);         // unsigned char to ASCII string as hex digits, 2 digits    
void uint16_hex_str(uint16_t, char *);       // uint16 to ASCII string as 4 hex digits
void uint32_hex_str(unsigned long, char *);  // uint32 to ASCII string as 8 hex digits

extern unsigned char day_of_week;
extern char sats_in_view[4];
extern char gps_altitude_str[32];  // ASCII
extern char gps_latitude_str[32];
extern char gps_longitude_str[32];
extern unsigned char nflag;

void lcd_altitude_feet(void);

const struct messages amsg1 = {0x80, 15, "GPS-Alt  Pr-Alt"};
const struct messages bmsg1 = {0x80, 9, "Device ID"};
const struct messages bmsg2 = {0xC3, 7, "BME-280"};
const struct messages bmsg3 = {0xC3, 7, "BMP-280"};
const struct messages bmsg4 = {0xC3, 7, "Unknown"};

extern void domsg(const struct messages *);
extern void set_reference(void);
extern void show_sats_leap(void);
//======================================
extern struct alltime time;

void get_local(void);
void start_local(void);
void test_local(void);
extern void test_sint16(int16_t, int16_t);  // for testing
uint16_t do_day_of_week(uint8_t, uint8_t, uint8_t);  // returns with string in nstr
extern void display_utc_uart(void);
//====================================
extern uint16_t zda_flag, gga_flag;
extern void show_zda_buffer(void); 
extern void show_gga_buffer(void);
extern void show_pubx_buffer(void);
extern void show_gga_buffer(void);
extern void decode_ZDA(uint16_t);
extern void decode_GGA(uint16_t);
//====================================
void show_stuff(void);