/* 
 Set Barometer Reference and also Local Offset amd DST
 Jed Margolin  9/17/2024
 Standard Pressure at sea level is 29.92 inches mercury = 1013.2 mb
 Ratio is 33.8636363636364    33.86363636 for float
 Enter in inches mercury because that is what everyone reports
*/  

#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "hardware/uart.h"
#include "hardware/structs/i2c.h"
#include "pico/binary_info.h"
#include <math.h>
#include "bref.h"

// Set Barometer Reference, Local UTC offset, DST enabled
void set_reference()
{
   uint16_t n;

   state = 0;

   SetCmdMode();    // set for commands
   lcd_byte(0x01);  // clear screen

   domsg(&msg1);   // "Set Reference"
   domsg(&msg2);   // "Inches Hg"

   sleep_ms(2000);

   clear_lcd(0xC0);

   get_bref_flash();
   show_pref();

//LED_break();
// HG reference is in 
   ref_hg_bcd = pref_work.pref_bcd;  // comes from bref flash
//   ref_hg_bcd = 0x2992;  // test: keep in bcd

   ref_lcd[0] = ( (ref_hg_bcd & 0xF000)>>12) + 0x30;
   ref_lcd[1] = ( (ref_hg_bcd & 0x0F00)>>8) + 0x30;
   ref_lcd[2] = '.';
   ref_lcd[3] = ( (ref_hg_bcd& 0x00F0)>>4) + 0x30;
   ref_lcd[4] = ( ref_hg_bcd&0x000F) + 0x30;

// Display on LCD
    lcd_wr_instr(0xC0);     // position
    for(n=0; n<5; n++)  { lcd_wr_data(ref_lcd[n]); }
    lcd_wr_data(0x22);  // inch

// Convert to hex
   ref_mb_hex = (ref_lcd[0] - 0x30) * 1000;
   ref_mb_hex = ref_mb_hex + (ref_lcd[1] - 0x30)* 100;
   ref_mb_hex = ref_mb_hex + (ref_lcd[3] - 0x30)* 10;
   ref_mb_hex = ref_mb_hex + (ref_lcd[4] - 0x30);

   strcpy(line, "\nHg: ");
   uint16_bcd_str(ref_mb_hex, nstr);       // unsigned int to BCD string
   strcat(line, nstr);
   strcat(line, "    mb: ");
//   uart_puts(UART_ID, line); 

   ref_mb_float = ref_mb_hex * 33.86363636;
   uint32_t temp = ref_mb_float;
   uint32_bcd_str(temp, nstr);       // unsigned int to BCD string
   strcat(line, nstr);
   strcat(line, "\n");
   uart_puts(UART_ID, line); 

// Display on LCD
   lcd_wr_instr(0xC7);     // position

   lcd_wr_data(nstr[4]);
   lcd_wr_data(nstr[5]);
   lcd_wr_data(nstr[6]);
   lcd_wr_data(nstr[7]);
   lcd_wr_data('.');
   lcd_wr_data(nstr[8]);
   lcd_wr_data(' ');
   lcd_wr_data('m');
   lcd_wr_data('b');

   sleep_ms(2000);

//   SetCmdMode();    // set for commands
//   lcd_byte(0x01);  // clear screen
//LED_break();

   state = 0;

//   ref_hg_bcd = 0x2992;  // keep in bcd
   ref_hg_bcd_save = ref_hg_bcd;  // keep in bcd

#if NO
   for(n = 0; n<4; n++)
   {
      display_hg(0x84, n);     // arrow position 0 - 3
      sleep_ms(2000);
   }
   LED_break();
#endif

   goback = 0;

   while(1)
   {
      if(irq_flag)   // 4 ms interrupts, debounces and processes the switches
      {
         irq_flag = 0;

         // Do the main state machine.
         (*state_ptr[state])();  // Use the state number as the index to the list of pointers to the state functions
      }  // end of irq_flag

      if(goback) { break;}

   } // end while

   return;
}   // end of set_reference()
//==========================================================
void display_hg(uint16_t pos, uint16_t arrow)     // arrow position 0 - 3
{
   uint16_t n;

   SetCmdMode();    // set for commands
   lcd_byte(0x01);  // clear screen

   ref_lcd[0] = ( (ref_hg_bcd & 0xF000)>>12) + 0x30;
   ref_lcd[1] = ( (ref_hg_bcd & 0x0F00)>>8) + 0x30;
   ref_lcd[2] = '.';
   ref_lcd[3] = ( (ref_hg_bcd& 0x00F0)>>4) + 0x30;
   ref_lcd[4] = ( ref_hg_bcd&0x000F) + 0x30;

// Display on LCD
    lcd_wr_instr(0x84);     // position
    for(n=0; n<5; n++)  { lcd_wr_data(ref_lcd[n]); }
    lcd_wr_data(0x22);  // inch

   if(arrow >1)  { arrow++;}
   lcd_wr_instr(pos + 0x40 + arrow);   // position on next line
   lcd_wr_data(0x5E);    // up arrow
}
//===============================
void state0()       
{
    display_hg(0x84, 3);     // arrow position 0 - 3

    if(switch1.press == 1)   // Increment
    {
       switch1.press = 0;    // reset the flag
       inc_bref_bcd0();
    }

    if(switch2.press == 1)   // Decrement
    {
       switch2.press = 0;    // reset the flag
       dec_bref_bcd0();
    }

    if(switch3.press == 1)   // Next
    {
       switch3.press = 0;    // reset the flag
       state = 1;
    }

    sleep_ms(500);
}
//------------------
void state1()       
{
    display_hg(0x84, 2);     // arrow position 0 - 3

    if(switch1.press == 1)   // Increment
    {
       switch1.press = 0;    // reset the flag
       inc_bref_bcd1();
    }

    if(switch2.press == 1)   // Decrement
    {
       switch2.press = 0;    // reset the flag
       dec_bref_bcd1();
    }

    if(switch3.press == 1)   // Next
    {
       switch3.press = 0;    // reset the flag

       state = 2;
    }

    sleep_ms(500);
}
//------------------
void state2()       
{
    display_hg(0x84, 1);     // arrow position 0 - 3

    if(switch1.press == 1)   // Increment
    {
       switch1.press = 0;    // reset the flag
       inc_bref_bcd2();
    }

    if(switch2.press == 1)   // Decrement
    {
       switch2.press = 0;    // reset the flag
       dec_bref_bcd2();
    }

    if(switch3.press == 1)   // Next
    {
       switch3.press = 0;    // reset the flag
       state = 3;
    }

    sleep_ms(500);
}
//------------------
void state3()       
{
    display_hg(0x84, 0);     // arrow position 0 - 3

    if(switch1.press == 1)   // Increment
    {
       switch1.press = 0;    // reset the flag
       inc_bref_bcd3();
    }

    if(switch2.press == 1)   // Decrement
    {
       switch2.press = 0;    // reset the flag
       dec_bref_bcd3();
    }

    if(switch3.press == 1)   // Next
    {
       switch3.press = 0;    // reset the flag
       state = 4;
    }

    sleep_ms(500);
}
//------------------
void state4()       
{
   clear_lcd(0xC0);
   domsg(&msg3);  // Save?  N/Y

   if(switch1.press == 1)   // Update Hg Reference
   {
      switch1.press = 0;    // reset the flag

      erase_bref_flash();
      do_B_reference(ref_hg_bcd);
      sleep_ms(100);

      SetCmdMode();         // set for commands
      lcd_byte(0x01);       // clear screen
      domsg(&msg4);         // "Updated"

      state = 5;            
   }

    if(switch2.press == 1)   // Hg not updated
    {
       switch2.press = 0;    // reset the flag

       SetCmdMode();         // set for commands
       lcd_byte(0x01);       // clear screen
       domsg(&msg5);         // "Ok, not changed"
       ref_hg_bcd = ref_hg_bcd_save;  // keep in bcd

       state = 5;            
    }

    if(switch3.press == 1)   // Next
    {
       switch3.press = 0;   // reset the flag
    }

    sleep_ms(1000);
}
//-----------------------------
void state5()     // End of Barometer setting
{
//   clear_lcd(0xC0);

   if(switch1.press == 1)
   {
      switch1.press = 0;   // reset the flag
   }

   if(switch2.press == 1)   // Not updated
   {
      switch2.press = 0;   // reset the flag
   }

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      local_wr = 0;

// Get offset and dst enable from Flash
      get_local_flash();
      state = 6;    // Now do Local Offset to UTC
   }
}
//-----------------------------
void state6()       // Local Offset from UTC
{
   uint16_t n;

   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines

   domsg(&msg7);         // "Local Offset"

// We have local offset in time.utc_offset and dst enable in time.dst_enable from Flash
   sint8_bcd_str(time.utc_offset, nstr);  // convert to string
   lcd_wr_instr(0xC0);  // position
   for(n=0; n<16; n++)
   {
      if(nstr[n] == 0)  {break;}
      lcd_wr_data(nstr[n]);
   }

   if(switch1.press == 1)
   {
      switch1.press = 0;   // increment
      local_wr = 1;
      time.utc_offset++;
//      if(utc_offset>12)  { utc_offset = 12; }
   }

   if(switch2.press == 1)   // decrement
   {
      switch2.press = 0;   // reset the flag
      local_wr = 1;
      time.utc_offset--;
      if(utc_offset<-12) { utc_offset = -12; }
   }

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      state = 7;    // Now do DST Enable
   }

   sleep_ms(500);
}
//-----------------------------
void state7()       // DST Observed
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   domsg(&msg8);     // "DST Observed"

   if(switch1.press == 1)
   {
      switch1.press = 0;   // reset the flag
      local_wr = 1;
      time.dst_observed = 1;
   }

   if(switch2.press == 1)   // Not updated
   {
      switch2.press = 0;   // reset the flag
      local_wr = 1;
      time.dst_observed = 0;
   }

   if(time.dst_observed)  { domsg(&msg9); }    // "Yes"
   else  { domsg(&msg10); }    // "No "

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      state = 8;    // Now do Update or Not
   }

   sleep_ms(500);
}
//-----------------------------
void state8()       // Save?
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   domsg(&msg11);     // "Save?  N/Y"

   if(switch1.press == 1)  // Yes
   {
      switch1.press = 0;   // reset the flag

      erase_sector(FLASH_LOCAL_SECTOR);
      do_local(time.utc_offset, time.dst_observed, 1);

      SetCmdMode();     // set for commands
      domsg(&msg4);  // "Updated"
      sleep_ms(1000);
      state = 9;
   }

   if(switch2.press == 1)   // Not updated
   {
      switch2.press = 0;   // reset the flag

      SetCmdMode();     // set for commands
      domsg(&msg5);  // "Ok, not changed"
      sleep_ms(1000);
      local_wr = 0;
      state = 9;
   }

   if(switch3.press == 1)   // Next
   {
      switch3.press = 0;   // reset the flag
      sleep_ms(500);
      state = 9;
   }

   sleep_ms(500);
}
//-----------------------------
void state9()       // Start over
{
   SetCmdMode();     // set for commands
   lcd_byte(0x01);   // clear lcd, both lines
   state = 0;
}
//=====================================
void inc_bref_bcd3()
{
   uint16_t temp;

   temp = ( (ref_hg_bcd & 0xF000)>>12) + 1;
   if(temp >0x09) { temp = 0;}
   ref_hg_bcd = (ref_hg_bcd & 0x0FFF) | temp<<12;
};

void dec_bref_bcd3()
{
   int16_t temp;

   temp = ( (ref_hg_bcd & 0xF000)>>12) - 1;
   if(temp <0x0) { temp = 0x09;}
   ref_hg_bcd = (ref_hg_bcd & 0x0FFF) | temp<<12;
};
//--------------------------
void inc_bref_bcd2()
{
   uint16_t temp;

   temp = ( (ref_hg_bcd & 0x0F00)>>8) + 1;
   if(temp >0x09) { temp = 0;}
   ref_hg_bcd = (ref_hg_bcd & 0xF0FF) | temp<<8;
};

void dec_bref_bcd2()
{
   int16_t temp;

   temp = ( (ref_hg_bcd & 0x0F00)>>8) - 1;
   if(temp <0x0) { temp = 0x09;}
   ref_hg_bcd = (ref_hg_bcd & 0xF0FF) | temp<<8;
};
//--------------------------
void inc_bref_bcd1()
{
   uint16_t temp;

   temp = ( (ref_hg_bcd & 0x00F0)>>4) + 1;
   if(temp >0x09) { temp = 0;}
   ref_hg_bcd = (ref_hg_bcd & 0xFF0F) | temp<<4;
};

void dec_bref_bcd1()
{
   int16_t temp;

   temp = ( (ref_hg_bcd & 0x00F0)>>4) - 1;
   if(temp <0x0) { temp = 0x09;}
   ref_hg_bcd = (ref_hg_bcd & 0xFF0F) | temp<<4;
};
//--------------------------
void inc_bref_bcd0()
{
   uint16_t temp;

   temp = ( (ref_hg_bcd & 0x000F)) + 1;
   if(temp >0x09) { temp = 0;}
   ref_hg_bcd = (ref_hg_bcd & 0xFFF0) | temp;
};

void dec_bref_bcd0()
{
   int16_t temp;

   temp = ( (ref_hg_bcd & 0x000F)) - 1;
   if(temp <0x0) { temp = 0x09;}
   ref_hg_bcd = (ref_hg_bcd & 0xFFF0) | temp;
};
//=========================================
void show_pref()
{
   uint16_t n;

// Send to uart from structure at pref_work
   strcpy(line, "\npref hg bcd hex: ");
   uint32_hex_str(pref_work.pref_bcd, nstr);    // unsigned int to BCD string
   strcat(line, nstr);
   strcat(line, "\n");
   uart_puts(UART_ID, line); 

   strcpy(line, "pref mb32: ");
   uint32_bcd_str(pref_work.pref_int32, nstr);  // unsigned int to BCD string
   strcat(line, nstr);
   strcat(line, "\n");
   uart_puts(UART_ID, line); 

   strcpy(line, "pref hg and mb string: ");
   strcat(line, pref_work.pref_str);
   strcat(line, "\n");
   uart_puts(UART_ID, line); 

   strcpy(line, "pref float: ");
   uint32_bcd_str(pref_work.pref_float, nstr);  // unsigned int to BCD string
   strcat(line, nstr);
   strcat(line, "\n");
   uart_puts(UART_ID, line); 

// Display string on LCD
   SetCmdMode();    // set for commands
   lcd_byte(0x01);  // clear screen

   domsg(&msg6);   // Barometric Ref
   lcd_wr_instr(0xC0);     // position
   for(n=0; n<6; n++)  { lcd_wr_data(pref_work.pref_str[n]); }
   for(n=8; n<19; n++)  { lcd_wr_data(pref_work.pref_str[n]); }
//LED_break();

   sleep_ms(4000);
}
//===============================
void make_pref(uint16_t hg_bcd)
{
   uint16_t ref_hg_hex;
   char ref_hg_str[32];
   char ref_mb_str[32];
   uint32_t ref_mb32;
//   struct pref pref_work;    // global
   
// Make hg string
   ref_hg_str[0] = ( (hg_bcd & 0xF000)>>12) + 0x30;
   ref_hg_str[1] = ( (hg_bcd & 0x0F00)>>8) + 0x30;
   ref_hg_str[2] = '.';
   ref_hg_str[3] = ( (hg_bcd& 0x00F0)>>4) + 0x30;
   ref_hg_str[4] = ( hg_bcd&0x000F) + 0x30;
   ref_hg_str[5] = 0x22; // quote
   ref_hg_str[6] = 0x00; // NULL

// Convert to hex uint16
   ref_hg_hex = (ref_hg_str[0] - 0x30) * 1000;
   ref_hg_hex = ref_hg_hex + (ref_hg_str[1] - 0x30)* 100;
   ref_hg_hex = ref_hg_hex + (ref_hg_str[3] - 0x30)* 10;
   ref_hg_hex = ref_hg_hex + (ref_hg_str[4] - 0x30);

// Convert to hg to mb (float)
   ref_mb_float = ref_hg_hex * 3.386363636;
   ref_mb32 = ref_mb_float;  // save as uint32
   uint32_bcd_str(ref_mb32, nstr);       // unsigned int to BCD string

// mb string
   ref_mb_str[0]= nstr[5];
   ref_mb_str[1] = nstr[6];
   ref_mb_str[2] = nstr[7];
   ref_mb_str[3] = nstr[8];
   ref_mb_str[4] = '.';
   ref_mb_str[5] = nstr[9];
   ref_mb_str[6] = ' ';
   ref_mb_str[7] = 'm';
   ref_mb_str[8] = 'b';
   ref_mb_str[9] = 0;

// Save in the structure
   strcpy(line, ref_hg_str);
   strcat(line, "   ");   
   strcat(line, ref_mb_str);
   strcpy(pref_work.pref_str, line);   // unsigned char pref_str[32];    // String, example: 29.92" Hg   1013.2 mb

   pref_work.pref_bcd = hg_bcd;              // uint16_t pref_bcd;
   pref_work.pref_float = ref_mb_float;      // float pref_float;
   pref_work.pref_int32 = ref_mb32;          // uint32_t pref_int32;

//   uart_puts(UART_ID, "New pref"); 
//   show_pref();  
//LED_break();
   sleep_ms(2000);
}
//===============================
