// Things that are common with other files
//=============================================
// GPIO Pin Assigments
#define LCD_E  20
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 22
#define LCD_D7 21

// Switches
#define PB1 14
#define PB2 13
#define PB3 12
#define SW 15    // slide switch, run or set reference barometric pressure
#define CHAN 3   // Header pin, for testing

// UART0 (on Header)
#define UART_TX_PIN 0
#define UART_RX_PIN 1
#define UART_ID uart0
#define BAUD_RATE 115200

// UART1 for GPS
#define UART_TX1_PIN 8
#define UART_RX1_PIN 9
#define UART_ID1 uart1
#define BAUD_RATE1 9600

#define LED1 2    // Green
#define LED2 27   // Red

#define BUF_LEN  33
#define ADC0_PIN 26
//======================================================
#define SECTOR_MAX 16  // sectors per block
#define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data

#define FLASH_SDATE_SECTOR 258     // 256 sectors for my log, 16 sectors per page
#define FLASH_SDATE_PAGE 4128     

#define FLASH_BREF_SECTOR 259      // Barometric Reference. sector for erase
#define FLASH_BREF_PAGE 4144       // Barometric Reference, page for reading and writing

#define FLASH_LOCAL_SECTOR 260     // Local Time Offset, dst enable
#define FLASH_LOCAL_PAGE 4160      // Local Time Offset, dst enable

#define FLASH_DST_SECTOR 261       // DST dates
#define FLASH_DST_PAGE 4176        // DST dates

#define FLASH_TEST_SECTOR 262      // for testing
#define FLASH_TEST_PAGE 4192       // for testing
//========================================================
#define BME_ID 0x76  // BMP280 device ID

#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6
//======================================================
// For reading, debouncing, and processing the switches
struct myswitch
{
    unsigned char debounce;     // debounce byte
    unsigned char state;        // debounced state of the switch
    unsigned char old;          // old state
    unsigned char press;        // switch has been pressed
    unsigned char release;      // switch has been released
};
//---------------------------------------------
// Messages
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};
//---------------------------------------------
// Time in different formats
struct alltime
{
    char utc_gps_hours[3];   // ASCII string
    char utc_gps_mins[3];    // ASCII
    char utc_gps_secs[3];    // ASCII
    char utc_gps_date[3];    // ASCII
    char utc_gps_month[3];   // ASCII
    char utc_gps_year[5];    // ASCII

    uint16_t utc_bcd_hours;           // bcd
    uint16_t utc_bcd_mins;            // bcd
    uint16_t utc_bcd_secs;            // bcd
    uint16_t utc_bcd_date;            // bcd
    uint16_t utc_bcd_month;           // bcd
    uint16_t utc_bcd_year;            // bcd

    uint16_t utc_hex_hours;           // hex (bin)
    uint16_t utc_hex_mins;            // hex (bin)
    uint16_t utc_hex_secs;            // hex (bin)
    uint16_t utc_hex_date;            // hex (bin)
    uint16_t utc_hex_month;           // hex (bin)
    uint16_t utc_hex_year;            // hex (bin)

    uint16_t utc_dow;                 // day of the week, 1 is sunday to 7 is saturday

    int16_t local_hex_hours;         // hex (bin)
    int16_t local_hex_mins;          // hex (bin)
    int16_t local_hex_secs;          // hex (bin)
    int16_t local_hex_date;          // hex (bin)
    int16_t local_hex_month;         // hex (bin)
    int16_t local_hex_year;          // hex (bin)

    uint16_t leapyear;               // is it a leapyear?
    int16_t utc_offset;
    uint16_t dst_observed;
    uint16_t dst_flag;

    uint16_t dst_start_date;         // dst starts in March
    uint16_t dst_end_date;           // dst ends in November
    char dst_start_date_str[22];
    char dst_end_date_str[22];
};
//---------------------------------------------
//Time-stamped log 148 bytes 
struct log
{
   uint16_t ts_number;     // index number                   0,1
   
   uint8_t ts_date;       // ASCII time-stamped date        2
   uint8_t ts_day;        // ASCII time-stamped day         3
   uint8_t ts_month;      // ASCII time-stamped month       4
   uint8_t ts_year;       // ASCII time-stamped year        5

   uint8_t ts_hours;      // ASCII time-stamped hour        6
   uint8_t ts_minutes;    // ASCII time-stamped minutes     7
   uint8_t ts_seconds;    // ASCII time-stamped second      8

   int32_t Ftemperature;  //  Hes                           9, 10, 11, 12
   uint32_t Pressure;     //  Hex                           13, 14 15, 16  
   uint32_t Humidity;     //  Hex                           17, 18, 19, 20
   char pressure_altitude_str[20];    // ASCII              21 - 39

   char sats[4];          // ASCII sats in view             40 - 44
   char gps_altitude_str[32];     // ASCII                  45 - 77
   char gps_latitude_str[32];     // ASCII                  78 - 110
   char gps_longitude_str[32];    // ASCII                  111 - 143

   // String, example: 29.92" Hg   1013.2 mb
   char pref_str[32];             // ASCII                  144 - 175

   uint8_t flag;          // flag for non-blank             176    (176 for uint32 boundry)
};
//===================================================
//bmedata 20 bytes 
struct data
{
   int32_t temperature_C;       //  Hex    0, 1, 2, 3
   int32_t temperature_F;       //  Hex    4, 5, 6, 7  
   uint32_t pressure;           //  Hex    8, 9, 10, 11
   uint32_t pressure_altitude;  //  Hex    12, 13, 14 15,  
   uint32_t humidity;           //  Hex    16, 17, 18, 19
};
//===================================================

