#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/spi.h"
#include "hardware/uart.h"
#include "hardware/structs/i2c.h"
#include "pico/binary_info.h"
#include "local.h"
#include "hardware/divider.h"
//========================================
void start_local()
{
   int16_t offset;
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint16_t n;
   uint8_t buffer[4];

// Start by getting the utc year so we can determine if it is a leap year and get the dst dates. 
// It might be a little off around the January 1 but you have to start somewhere. At least the year will be correct 

// These are in Flash
   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_LOCAL_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<4; n++)  { buffer[n] = *p++; }
   memcpy(&mylocal, buffer, 4);
   time.utc_offset = mylocal.utc_offset;
   time.dst_observed = mylocal.dst_enable;

//   offset = -8;          // default time zone
   offset = time.utc_offset;

   if(offset<0)  { get_local_minus(offset); }   // when UTC offset is negative
   else          { get_local_plus(offset);  }   // when UTC offset is positive

   display_utc_lcd();
   sleep_ms(2000);
   display_local();

// Now we can do leapyear and dst
   find_dst(time.local_hex_year);
   time.dst_flag = is_dst();      // Are we in DST?
   strcpy(line, time.dst_start_date_str);
   strcat(line,"   ");
   strcat(line, time.dst_end_date_str);
   if(time.dst_flag)  { strcat(line,"   DST Yes"); }
   else               { strcat(line,"   DST No"); }
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);   // send it out

//   time.dst_observed = 1;
//   time.dst_flag = 1;
   if( (time.dst_observed) && ( time.dst_flag) )   { offset = offset + 1; }

   if(offset<0)  { get_local_minus(offset); }  // when UTC offset is negative
   else          { get_local_plus(offset);  }  // when UTC offset is positive
   display_local();  // Display local Time on lcd and send through uart

// Now we have: time.local_hex_secs, time.local_hex_mins, time.local_hex_hours, time.local_hex_month, 
// time.local_hex_date, time.local_hex_month, time.local_hex_year, time.dst_start_date, time.dst_start_date_str,
// time.dst_end_date, time.dst_end_date_str 
}
//==========================================================
// Operates on utc hours, mins, secs, month, date, year
// time.dst_start_date, time.dst_end_date, time.dst_observed, time.dst_flag
void get_local()
{
   int16_t offset;

   offset = time.utc_offset;
   if( (time.dst_observed) && ( time.dst_flag) )   { offset = offset + 1; }

   if(offset<0)  { get_local_minus(offset); }  // when UTC offset is negative
   else          { get_local_plus(offset);  }  // when UTC offset is positive
   display_local();  // Display local Time on lcd and send through uart
}
//==========================================================
// Get local time for negative UTC offset
// We already have year, month, day, hour in binary
// For Local Time
// int16_t local_year, local_month, local_date, local_hours, local_mins, local_secs;
// unsigned char leapyear;
void get_local_minus(int16_t offset)   
{
   // minutes and seconds are the same
   time.local_hex_secs = time.utc_hex_secs;
   time.local_hex_mins = time.utc_hex_mins;

//test_sint16(1, 1234);
//test_sint16(2, -1234);
//test_sint16(3, time.local_hex_hours);

   time.local_hex_hours = time.utc_hex_hours + offset;    // utc hour + offset
//test_sint16(2, time.utc_hex_hours);
//test_sint16(3, time.local_hex_hours);

   if(time.local_hex_hours >= 0)  // Same day; we don't have to adjust days and maybe months and maybe years
   { 
      time.local_hex_date = time.utc_hex_date;
      time.local_hex_month = time.utc_hex_month;
      time.local_hex_year = time.utc_hex_year;
//      uart_puts(UART_ID, "Same day\n");
      return; 
   } 

   // else: now we have to decrement the day

   // first add 24 to the hour since it is the previous utc day
   time.local_hex_hours = time.local_hex_hours + 24;

   // get the new date
   time.local_hex_date = time.utc_hex_date - 1;   // subtract a day
//test_sint16(5, time.local_hex_date);

   if(time.local_hex_date >= 1)      // we don't have to adjust months and maybe years
   { 
      time.local_hex_month = time.utc_hex_month;
      time.local_hex_year = time.utc_hex_year;
      return; 
   }

   // else, shazbot, the day was zero so we have to adjust the month
 
   // get the days in the previous month. but first
   time.local_hex_month = time.utc_hex_month - 1;

   if(time.local_hex_month != 0)  // we are good
   {
      time.local_hex_year = time.utc_hex_year;

      // now do the last day of the previous month
      time.leapyear = is_leap(time.utc_hex_year);            // is it a leap year?
      if(time.leapyear)  { time.local_hex_date = months_ly[time.local_hex_month]; }
      else               { time.local_hex_date = months[time.local_hex_month]; }

      return;
   }

   //  else, it is december and rollback the year   
   time.local_hex_month = 12;
   time.local_hex_date = 31;
   time.local_hex_year = time.utc_hex_year - 1;  

   save_year = time.local_hex_year;
   return;
} // end of adjusting for local time which might have required adjusting the day, month, and year
//=====================================================
// Get local time for positive UTC offset
// We already have year, month, day, hour in binary
// For Local Time
// int16_t local_year, local_month, local_day, local_hour, local_minute, local_second;
// unsigned char leapyear;
void get_local_plus(int16_t offset)   
{
   int16_t last_day;

   // minutes and seconds are the same
   time.local_hex_secs = time.utc_hex_secs;
   time.local_hex_mins = time.utc_hex_mins;

   time.local_hex_hours = time.utc_hex_hours + offset;    // utc hour + offset
   if(time.local_hex_hours < 24)  // Same day; we don't have to adjust days and maybe months and maybe years
   { 
      time.local_hex_date = time.utc_hex_date;
      time.local_hex_month = time.utc_hex_month;
      time.local_hex_year = time.utc_hex_year;
      return; 
   } 

   // else, now we have to increment the day

   // first subtract 24 to the hour since it is the next utc day
   time.local_hex_hours = time.local_hex_hours - 24;

   // get the new date
   time.local_hex_date = time.utc_hex_date + 1;   // add a day

   // is it past the last day of the month?
   time.leapyear = is_leap(time.utc_hex_year);            // is it a leap year?
   if(time.leapyear)  { last_day = months_ly[time.utc_hex_month]; }
   else               { last_day = months[time.utc_hex_month]; }

   if(time.utc_hex_date <= last_day)      // we don't have to adjust months and maybe years
   { 
      time.local_hex_month = time.utc_hex_month;
      time.local_hex_year = time.utc_hex_year;
      return; 
   }

   // else, shazbot, the day was zero so we have to adjust the month
 
   // get the days in the month. but first
   time.local_hex_month = time.utc_hex_month + 1;

   if(time.utc_hex_month <= 12)  // we are good
   {
      time.local_hex_year = time.utc_hex_year;

      // now do the first day of the next month
      time.local_hex_date = 1;

      return;
   }

   //  else, it is past december so increment the year   
   time.local_hex_month = 1;
   time.local_hex_date = 1;
   time.local_hex_year = time.utc_hex_year + 1;  

   save_year = time.local_hex_year;
   return;
} // end of adjusting for local time which might have required adjusting the day, month, and year
//=====================================================
void display_local()   // current (local) time
{
   uint8_t weekday, temp;
   uint16_t m;

   SetCmdMode();      // set for commands
   lcd_byte(0x01);    // clear screen

   strcpy(line,"Local Time: ");
//   time12();   // Adjust if 12 hour time

// Hours:Minutes:Seconds
//   uint8_bcd_str2(time.local_hex_hours, nstr);   // unsigned char to ASCII string, 2 digits
   do_clock8(time.local_hex_hours);
   strcat(line,nstr);
   lcd_wr_instr(0x80);        // position of time
   lcd_wr_data(nstr[0]);
   lcd_wr_data(nstr[1]);

   strcat(line,":");
   lcd_wr_data(':');

//   uint8_bcd_str2(time.local_hex_mins, nstr);  // unsigned char to ASCII string, 2 digits
   do_clock8(time.local_hex_mins);
   strcat(line,nstr);
   lcd_wr_data(nstr[0]);
   lcd_wr_data(nstr[1]);

   strcat(line,":");
   lcd_wr_data(':');

//   uint8_bcd_str2(time.local_hex_secs, nstr);  // unsigned char to ASCII string, 2 digits
   do_clock8(time.local_hex_secs);
   strcat(line,nstr);
   lcd_wr_data(nstr[0]);
   lcd_wr_data(nstr[1]);

   strcat(line,"   ");
   lcd_wr_instr(0xC0);        // position of date

// Month/Date/Year
//   uint8_bcd_str2(time.local_hex_month, nstr);   // unsigned char to ASCII string, 2 digits
   do_clock8(time.local_hex_month);
   strcat(line,nstr);
   lcd_wr_data(nstr[0]);
   lcd_wr_data(nstr[1]);

   strcat(line,"/");
   lcd_wr_data('/');

//   uint8_bcd_str2(time.local_hex_date, nstr);   // unsigned char to ASCII string, 2 digits
   do_clock8(time.local_hex_date);
   strcat(line,nstr);
   lcd_wr_data(nstr[0]);
   lcd_wr_data(nstr[1]);

   strcat(line,"/");
   lcd_wr_data('/');

//   uint16_bcd_str4(time.utc_hex_year, nstr);   // uint16 to ASCII string, 4 digits
   uint16_bcd_str4(time.local_hex_year, nstr);   // uint16 to ASCII string, 5 digits
   strcat(line,nstr);
   lcd_wr_data(nstr[0]);
   lcd_wr_data(nstr[1]);
   lcd_wr_data(nstr[2]);
   lcd_wr_data(nstr[3]);
   
   strcat(line,"   ");

// Day name
   weekday = do_day_of_week(time.local_hex_month, time.local_hex_date,time.local_hex_year-2000);     // day of the week
   weekday--;  // day 1 is entry 0
   lcd_wr_instr(0xCC);
   for(m=0;m<3;m++)
   {
      temp =  day_array[weekday][m];
      nstr[m] = temp;
      lcd_wr_data(temp);
   }
   nstr[3] = 0;

   strcat(line,nstr);
   strcat(line,"\n\n");

   domsg(&lmsg3);    // "Local"
   uart_puts(UART_ID, line); 

   if(!hours12) {domsg(&lmsg3);}    // "Local"
   else
   {
      if(pmflag)  {domsg(&lmsg13);}  // "PM"
      else {domsg(&lmsg12);}         // "AM"
   }
   
   sleep_ms(3000);
}
//================================================
// Is it a leap year?
//   The year must be evenly divisible by 4
//   But not if it is evenly divisible by 100 unless it is evenly divisible by 400
uint16_t is_leap(uint16_t Lyear)
{
//   Lyear = 2100;    // for testing

   if( (Lyear & 0x003) != 0)
   { return(0);}                 // it is not evenly divisible by 4
// it is evenly divided by 4

   if(Lyear % 400 == 0)   // it is evenly divisible by 400 (remainder is zero)
   { return(1);}
//  it was not evenly divisible by 400

   if(Lyear % 100 == 0)  {return(0);}   // it is otherwise evenly divisible by 100

// it was evenly divisible by 4, not a century year unless it was 400 year century
   return(1);
}
//================================================
uint16_t is_leap2(uint16_t year)    // years in binary
{
   if( (year & 0x003) != 0)
   { return(0);}                 // it is not evenly divisible by 4
// it is evenly divided by 4

   if(year % 400 == 0)   // it is evenly divisible by 400 (remainder is zero)
   { return(1);}
//  it was not evenly divisible by 400

   if(year % 100 == 0)  {return(0);}   // it is otherwise evenly divisible by 100

// it was evenly divisible by 4, not a century year unless it was 400 year century
   return(1);
}
//==========================================================
// Get day of the week
uint16_t get_day(uint16_t month, uint16_t date, uint16_t year)
{
   uint16_t year2d;
   uint16_t temp;

// monthkey is in main.h
// const unsigned int monthkey[12] = {1,4,4,0,2,5,0,3,6,1,4,6};  // month's key value

// Last two digits of the year
// x = y * 10 = y * (8 + 2) = y*8 + y*2 = y<<3 + y<<1
   year2d =  year - 2000;
   temp = year2d>>2;   // last two digits of the year; divide by 4; drop the remainder
   temp = temp + date;  // Add the day of the month
   temp = temp + monthkey[month-1];  // Add the month's key value, from the table.

// If the date is in January or February of a leap year, subtract 1.
   if( (month == 1 ) | (month == 2) )
   {
      if( is_leap(year) )  { temp = temp-1;}
   }

   temp = temp + 6;       // add the century code (good until 2400); 0 for 1900s
   temp = temp + year2d;  // Add the last two digits of the year
   temp = temp % 7;       // divide by 7; use the remainder which is the day of the week

   return(temp);
}
//===================================================
unsigned char ldec_bcd(unsigned char time, unsigned char min, unsigned char max)
{
    unsigned char bcd0, bcd1;

    if(time == min)
    {
        return(max);
    }

    bcd0 = time & 0x0F;
    bcd1 = time>>4;

    bcd0--;
    if(bcd0 == 0xFF)
    {
       bcd0 = 0x09;
       bcd1 = bcd1 - 1;
    }

    time = bcd0 | (bcd1<<4);
    return(time);
}
//===================================================
unsigned char linc_bcd(unsigned char time, unsigned char min, unsigned char max)
{
    unsigned char bcd0, bcd1;

    if(time == max)
    {
        time = min;
        return(time);
    }

    bcd0 = time & 0x0F;
    bcd1 = time>>4;

    bcd0++;
    if(bcd0 == 10)
    {
       bcd0 = 0;
       bcd1 = bcd1 + 1;
    }

    time = bcd0 | (bcd1<<4);

    return(time);
}
//==========================================================
// Adjust if 12 hour time
void time12()
{
   if(hours12)
   {
      if(local_hour<12)
      {
         pmflag = 0;
         return;
      }

      if(local_hour == 0)  //12 AM
      {
         local_hour = 12;
         pmflag = 0;
         return;
      }

      if(local_hour == 12)
      {
         pmflag = 1;
         return;
      }

// It must be afternoon
      local_hour = local_hour - 12;
      pmflag = 1;
   }
}
//==========================================================
void find_dst(uint16_t year)
{
// DST starts on the second Sunday in March
// Find day of the week for March 1
   unsigned char day;

   day = get_day(3,1,year);   // 0 is Saturday
   time.dst_start_date = dst_start_table[day];      // use a table

   day = get_day(11,1,year);                  // use a table
   time.dst_end_date = dst_end_table[day];

   strcpy(line,"DST starts 3/");
   uint8_bcd_str2(time.dst_start_date, nstr);   // unsigned char to ASCII string, 2 digits
   strcat(line, nstr);
   strcat(line, "/");
   uint16_bcd_str4(year, nstr);   // uint16 to ASCII string, 4 digits
   strcat(line, nstr);
 //  strcat(line,"\n");
   strcpy(time.dst_start_date_str, line);

   strcpy(line,"DST ends 11/");

   uint8_bcd_str2(time.dst_end_date, nstr);  // unsigned char to ASCII string, 2 digits
   strcat(line, nstr);
   strcat(line, "/");
   uint16_bcd_str4(year, nstr);   // uint16 to ASCII string, 4 digits
   strcat(line, nstr);
//   strcat(line,"\n");
   strcpy(time.dst_end_date_str, line);
}
//==================================================
// Are we in DST?
// For Local Time
// int16_t local_year, local_month, local_day, local_hour, local_minute, local_second;
// unsigned char leapyear;
unsigned char is_dst()
{
   if(!time.dst_observed)    // If DST is not observed then we already know the answer is No
   { return(0); }

// Are we past March and before November?
   if( (time.local_hex_month >3) && (time.local_hex_month<11))   // we are definitely in the DST window
   { return(1); }

// Are we before March or after November?
   if( (time.local_hex_month <3) | (time.local_hex_month>11))   // we are definitely not in the DST window
   { return(0); }

// If we are in March are we before the start date and time of DST for the year?
   if( (time.local_hex_month == 3) )
   {
      if( (time.local_hex_date< time.dst_start_date) ) { return(0); }   // not in the window
      if( (time.local_hex_date> time.dst_start_date) ) { return(1); }   // in the window

      // We are in the day when 2AM becomes 3AM
      if( (time.local_hex_hours < 02) )
      { return(0); }                // not in the window
      if( (time.local_hex_hours >= 02) )
      { return(1); }                // We are in the window
      //  Since the time changed from 2AM to 3AM we don't have to remember it changed
   }

// If we are in November are we before the end date and time of DST for the year?
   if( (time.local_hex_month == 11) )
   {
      if( (time.local_hex_date < time.dst_end_date) ) { return(1); }   // still in the window
      if( (time.local_hex_date > time.dst_end_date) ) { return(0); }   // not in the window

      // this is the day and we have to know if we have already changed it or 1AM-2AM will be wrong
      if( (time.local_hex_hours < 01) )  { return(1); }    // still in the window
      if( (time.local_hex_hours > 02) )  { return(0); }    // not in the window

      // we are now between 1AM and 2AM
//      if(!dst_window) { return(0);}  // we already turned the window off

      // now we are waiting for 2 AM or later
      if( (time.local_hex_hours >= 02) ) { return(0); }   // We are no longer in the window (and will remember it in dst_window which is global)
   }

   return(0);   // should never get here
}
//================================================
void test_sint16(int8_t n, int16_t var)
{
   uint8_bcd_str(n, line);  // unsigned char to ASCII string, 3 digits
   strcat(line," Test: ");
   sint16_bcd_str(var, nstr);  // int16 to ASCII string, 5 digits
   strcat(line,nstr);
   strcat(line,"\n");
   uart_puts(UART_ID, line);
}
//================================================
void test_local()
{
   int16_t offset;
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint16_t n;
   uint8_t buffer[4];

// Start by getting the utc year so we can determine if it is a leap year and get the dst dates. 
// It might be a little off around the January 1 but you have to start somewhere. At least the year will be correct 

// These are in Flash
   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_LOCAL_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<4; n++)  { buffer[n] = *p++; }
   memcpy(&mylocal, buffer, 4);
   time.utc_offset = mylocal.utc_offset;
   time.dst_observed = mylocal.dst_enable;

// For now 
//   offset = -8;          // default time zone
   offset = time.utc_offset;
//test_sint16(1, offset);
//test_sint16(11, time.dst_observed);

   if(offset<0)  { get_local_minus(offset); }   // when UTC offset is negative
   else          { get_local_plus(offset);  }   // when UTC offset is positive
   display_local();

// Now we can do leapyear and dst
   find_dst(time.local_hex_year);
   time.dst_flag = is_dst();      // Are we in DST?
//test_sint16(15, time.dst_flag);
   strcpy(line, time.dst_start_date_str);
   strcat(line,"   ");
   strcat(line, time.dst_end_date_str);
   if(time.dst_flag)  { strcat(line,"   DST Enabled"); }
   else               { strcat(line,"   DST Disabled"); }
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);   // send it out

//   time.dst_observed = 1;
//   time.dst_flag = 1;

   if( (time.dst_observed) && ( time.dst_flag) )   { offset = offset + 1; }

   if(offset<0)  { get_local_minus(offset); }  // when UTC offset is negative
   else          { get_local_plus(offset);  }  // when UTC offset is positive
   display_local();  // Display local Time on lcd and send through uart

// Now we have: time.local_hex_secs, time.local_hex_mins, time.local_hex_hours, time.local_hex_month, 
// time.local_hex_date, time.local_hex_month, time.local_hex_year, time.dst_start_date, time.dst_start_date_str,
// time.dst_end_date, time.dst_end_date_str 
}
//======================================================
// for the clock convert hex to ASCII string
void do_clock8(uint8_t value)
{
   uint16_t n;

   n = value<<1;

   nstr[0] = clock8[n++];
   nstr[1] = clock8[n++];
   nstr[2] = 0;  // NULL
}
//======================================================
void test1_clock8()
{
   uint16_t m, n;

   m = 0;
   uart_puts(UART_ID, "Test Clock 8");    // send it out
   strcpy(line,"\n");
   for(n=0; n<60; n++)
   {
      do_clock8(n);
      strcat(line, nstr);
      strcat(line, "  ");
      if(++m == 10)  
      { 
         m = 0; 
         uart_puts(UART_ID, line);   // send it out
         strcpy(line,"\n");
      }
   }
   uart_puts(UART_ID, "\n\n");    // send it out
}
//======================================================
void test2_clock8()
{
   uint16_t n;

   n = 23;
   while(1)
   {
      gpio_put(CHAN,1);  
      do_clock8(n);      // 192 ns
      gpio_put(CHAN,0);   
      uint8_bcd_str2(n, nstr);  // unsigned char to ASCII string, 2 digits   2.05 us
   }
}
//======================================================
void test16()
{
   uint16_hex_bin_str(12345, nstr);

   strcpy(line,"\nTest Hex to ASCII: ");
   strcat(line,nstr);
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);   // send it out

   while(1)
   {
      gpio_put(CHAN,1);  
      uint16_hex_bin_str(12345, nstr);  // 3.9 us
      gpio_put(CHAN,0);
      uint16_hex_str(12345, nstr);      // 436 ns
   }
}
//--------------------------------------------------
//void my_itoa(unsigned int bin,char *str)
void uint16_hex_bin_str(uint16_t value, char *str)
{
   #define decimal 5
   uint32_t factor;
   uint16_t remains;
   uint8_t bcd[8];
   uint8_t m, n;

//   value = 65535;
   
   remains = value;
   for(n=0; n<decimal; ++n)
   {
      factor = 1;
      for (m=0; m<(decimal - n - 1); ++m)
      factor *= 10;
      bcd[decimal - n - 1] = remains / factor;
      remains = remains % factor;
    }

   str[0] = bcd[4] + 0x30;
   str[1] = bcd[3] + 0x30;
   str[2] = bcd[2] + 0x30;
   str[3] = bcd[1] + 0x30;
   str[4] = bcd[0] + 0x30;
   str[5] = 0;

/*
   strcpy(line,"\nTest Hex to ASCII: ");
   strcat(line,nstr);
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);   // send it out
*/
}
//--------------------------------------------------
//==========================================
