// Flash Memory

#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "hardware/uart.h"
#include <string.h>
#include <hardware/flash.h>
#include <hardware/sync.h>
#include "myflash.h"  // mine
//================================================================
// Flash Memory Log
void do_flash()
{
   uint16_t n;

   uart_flag = 0;
   erase_flag = 0;

// make sure uart FIFO is empty, disable FIFO and leave it disabled
   uart_set_fifo_enabled(UART_ID, false);
   for(n=0; n<32; n++)   
   {
       // keep going until FIFO is empty (0 bytes in FIFO)
       if( uart_is_readable(UART_ID) )   { uart_char = uart_getc(UART_ID); }
       else {break;}
   }

   uart_puts(UART_ID, "Keyboard Commands (Upper or Lower Case)\n");
   uart_puts(UART_ID, "   V: Show Version\n");
   uart_puts(UART_ID, "   B: Show BME-280\n");
   uart_puts(UART_ID, "   O: Log BME-280 Once\n");
   uart_puts(UART_ID, "   L: Log BME-280 Every 5 minutes\n");
   uart_puts(UART_ID, "   X: Stop Logging BME-280\n");
   uart_puts(UART_ID, "   D: Dump All of the Flash Logs to uart\n");
   uart_puts(UART_ID, "   E: Erase Data Flash?  (then Y/N)\n"); 
   uart_puts(UART_ID, "   S: Reset Start Date?  (then Y/N)\n");
   uart_puts(UART_ID, "   P: BME-280 Parameters\n");
   uart_puts(UART_ID, "   H: Repeat This List\n\n"); 

   uart_puts(UART_ID, "> Ready For Commands\n\n"); 

//LED_break();
}
//===================================================================
void find_first_blank_page()
{  
   uint16_t m, n;

// Find the first blank page
   for(n=0; n<4096; n++)
   {
      if(is_page_blank(n))  { break; }    // 256 bytes per page
   }   

   Page_Index = n;
   Sector_Index = Page_Index>>4;   // referenced to erase sector, 16 pages per sector (256*16 = 4096);

   strcpy(line,"First Blank Page Index: ");
   my_itoa(Page_Index,nstr);  // ends with NULL
   strcat(line,nstr);

   strcat(line,"    First Blank Sector Index: ");
   my_itoa(Sector_Index,nstr);  // ends with NULL
   strcat(line,nstr);
   strcat(line,"\n");
   uart_puts(UART_ID,line);

// Are the remaining pages blank?
   first_time_flash = 1;    // first time the Flash has been used
   if(n == 4095)  { first_time_flash = 0; }
   else 
   {
      for(m=n; m<4096; m++)   // pick up where we left off
      {
         if(!is_page_blank(m))    // not blank?
         { 
           first_time_flash = 0;
//           show_page(m);
           break;   // breaks the 'for' loop
         }
//         show_page(m);

      }
   }
  
   if(first_time_flash)  { uart_puts(UART_ID,"First Time Through Flash\n"); }
   else                  { uart_puts(UART_ID,"Not First Time Through Flash\n"); }
//LED_break();
}
//====================================================
uint16_t is_page_blank(uint32_t page)
{   
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data

   uint16_t n;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t data;
   uint16_t blank;

   blank = 1;    // blank
   myflash_read_address = XIP_BASE + (512 * 1024) + (page<<8);
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<256; n++)   
   {
      data = *p++;
      if(data != 0xFF)   
      { 
         blank = 0; 
         break;
      }
   }

   return blank;
}
//==========================================================
void show_page(uint32_t page)
{   
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data

   uint16_t n;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t data;
  
   myflash_read_address = XIP_BASE + (512 * 1024) + (page<<8);
   p = (uint8_t *)myflash_read_address;

   strcpy(line,"Page: "); 
   my_itoa(page,nstr);  // ends with NULL
   strcat(line,nstr);
   strcat(line,"\n");
   uart_puts(UART_ID,line);

   for(n=0; n<256; n++)   
   {
      data = *p++;

      my_hextoa2(data, line); 
      strcat(line,"  ");   
      uart_puts(UART_ID, line);
   }
   uart_puts(UART_ID,"\n\n");
}
//============================================
// Convert integer to ASCII string for the uart
void my_itoa(unsigned int bin,char *str)
{
// First Convert Binary to ASCII

// bin = 1234; // 0x67 = 103d; 0x386 = 902d
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = 0;  // NULL
}
//===============================================
// Convert integer to ASCII string for the uart; suppress first two leadimg zeroes
void my_itoa2(unsigned int bin,char *str)  // remove leading zeros; end with NULL
{
// First Convert Binary to ASCII

// bin = 1234; // 0x67 = 103d; 0x386 = 902d
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string
// Display it
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = 0;
   }
   else
   { 
      if(char2 != 0x30)
      { 
         str[0] = char2;
         str[1] = char3;
         str[2] = char4;
         str[3] = 0;
      }
      else
      {   
         if(char3 != 0x30)
         { 
            str[0] = char3;
            str[1] = char4;
            str[2] = 0;
         }
         else
         {
            str[0] = char4;
            str[1] = 0;
         }
      }
   }
}
//===============================================
// Fills a string with two ASCII numbers from two BCD numbers; ends with NULL if desired
void line_fill(unsigned char hex, char *ptr, unsigned char index, unsigned char end)
{
   unsigned char digit1, digit2;

   digit1 =  hex>>4;
   digit2 = hex & 0x0F;

   if(digit1<10)
      {digit1 = digit1 + 0x30;}
   else 
      { digit1 = digit1 + 0x37;}

   if(digit2<10)  
      {digit2 = digit2 + 0x30;}
   else 
      { digit2 = digit2 + 0x37;}

      ptr[index++] = digit1;
      ptr[index] = digit2;
      if(end) { ptr[++index] = 0; }  // NULL
}
//================================================      
// Fills a string with two ASCII numbers from a binary byte; ends with NULL if desired
void line_fill_bin(unsigned char bin,char *ptr, unsigned char index, unsigned char end)
{
// First Convert unsigned hex byte to BCD 
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char char1, char2;
   unsigned char hcarry;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
//          hcarry = 1;
      }
      else {hcarry = 0;}
   }

// Convert to ascii
   char1 = bcd1 + 0x30;
   char2 = bcd0 + 0x30;

// Put it in the string
   ptr[index++] = char1;
   ptr[index] = char2;
   if(end) { ptr[++index] = 0; }  // NULL
}
//====================================================
// Hex byte to string
void my_hextoa2(unsigned char bin,char *str) 
{
// First Convert Binary to ASCII
   unsigned char hex0;
   unsigned char hex1;
   unsigned char char1, char2;

   hex0 = bin>>4;
   hex1 = bin&0x0F;

// Convert to ascii
   if(hex0<10)
      {char1 = hex0 + 0x30;}
   else 
      { char1 = hex0+ 0x37;}

   if(hex1<10)
      {char2 = hex1 + 0x30;}
   else 
      { char2 = hex1+ 0x37;}

// Make the string
      str[0] = char1;
      str[1] = char2;
      str[2] = 0;  // NULL
}
//====================================================
// long int hex to string, space between ints
void my_hextoa8(uint32_t bin, char *hex) 
{
// Convert Binary to ASCII
   unsigned int m, n;
   char temp[10];

   m = 7;
   for(n=0; n<8; n++)
   {
      temp[m] = bin & 0x0F;
      m--;
      bin = bin>>4;
   }
   asm(" nop");    // for emulator

   // Convert to ascii
   for(n=0; n<8; n++)
   {
     if(temp[n]<10)  { temp[n] = temp[n] + 0x30; }
     else            { temp[n] = temp[n] + 0x37; }
   }
  
// Format the string
   for(n=0; n<4; n++)  { hex[n] = temp[n]; }
   hex[4] = ' ';

   for(n=4; n<8; n++)  { hex[n+1] = temp[n]; }
   hex[9] = 0x00;  // NULL   Now it is a string

   asm(" nop");    // for emulator
}
//================================================
// Log the time
void log_time()
{
// ts_number is the log number starting from Reset
   my_itoa(ts_number,nstr); // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

//   i2c_get_time();

   line_fill(time_buffer[MONTH], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[DATE], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[YEAR], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(time_buffer[HOURS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[MINS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[SECS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");   // ends with a NULL
}
//========================================================
void write_flash(unsigned char *buffer, uint32_t myflash_target_offset)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256  
   
//   uint32_t myflash_target_offset;
   uint32_t myflash_read_address;
   char *p;

   unsigned char buffer_in[256];
   unsigned int n;

//   myflash_target_offset = FLASH_TARGET_OFFSET;            // 512 * 1024;
//   myflash_read_address = XIP_BASE + FLASH_TARGET_OFFSET;  // ZIP_BASE + (512 * 1024);
   myflash_read_address = XIP_BASE + myflash_target_offset;  // ZIP_BASE + (512 * 1024);

// write   
   flash_range_program(myflash_target_offset, buffer, FLASH_PAGE_SIZE);

// read
//   char *p = (char *)XIP_BASE + (512 * 1024);
   p = (char *)myflash_read_address;     // starts at: XIP_BASE + (512 * 1024);

   for(n=0; n<256; n++)
   {
      buffer_in[n] = *p++;
   }

// Send it out through uart
   for(n=0; n<256; n++)
   {
         my_hextoa2(buffer_in[n],nstr);
         strcpy(line, nstr);
         strcat(line,"  ");
         uart_puts(UART_ID, line);
   }
   uart_puts(UART_ID, "\n\n");
}
//============================================
void read_page(uint16_t page, uint16_t number)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_target;
   uint32_t myflash_read_address;
   char *ptr;
   unsigned int m, n;

   myflash_target = FLASH_TARGET_OFFSET + (page<<8);
   myflash_read_address = XIP_BASE + myflash_target;  
   
   strcpy(line,"\nFlash Page: ");
   my_itoa(page, nstr);
   strcat(line, nstr);
   strcat(line,"\n");
   uart_puts(UART_ID, line);

// read the page
   ptr = (char *)myflash_read_address;
 
/*
   for(n=0; n<number; n++)   
   { 
      cdata = *ptr++;
      my_hextoa2(cdata, line); 
      strcat(line,"  ");   
      uart_puts(UART_ID, line);
   }
   uart_puts(UART_ID, "\n\n");
*/

   m = 32;
   line[0] = 0;  // Start with NULL
   for(n=0; n<256; n++)
   {
      my_hextoa2(*ptr++, nstr);
      strcat(line,nstr);   

      if(--m == 0)
      {
         strcat(line, "\n");
         uart_puts(UART_ID, line); 
         m = 32;
         line[0] = 0;  // start new line
      }
      else  { strcat(line,"  "); }   
   }
   strcat(line,"\n\n");
   uart_puts(UART_ID, line); 

}
//===================================================
// erase all target sectors
void erase_all_sectors()
{
   uint16_t n;

   for(n=0; n<256; n++)  { erase_sector(n);}
   Page_Index = n;
   Sector_Index = 0;
   Page_Index = 0;
}
//---------------------------------------
void erase_sector(uint16_t sector)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_target;
   uint32_t myflash_read_address;
   char *p;

   unsigned char buffer_in[4096], flag;
   unsigned int n;

   myflash_target = FLASH_TARGET_OFFSET + (sector<<12);
   myflash_read_address = XIP_BASE + myflash_target;  

   ints = save_and_disable_interrupts();

// erase
   flash_range_erase(myflash_target, FLASH_SECTOR_SIZE);

   restore_interrupts(ints);

// check that the sector is blank (all ones)
   flag = 0;
   p = (char *)myflash_read_address;
   for(n=0; n<4096; n++)
   {
      buffer_in[n] = *p++;
      if(buffer_in[n] != 0xFF)  { flag = 1; }
   }

   my_itoa(sector, nstr);
   strcpy(line, nstr);
   strcat(line,": ");
   if(flag) { strcat(line,"Sector not blank:\n");}
   else     { strcat(line,"Sector is blank:\n");}
 
   uart_puts(UART_ID, line);
}
//==========================================
void myflash_erase_sector(uint16_t sector)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
 
   uint32_t myflash_erase_address;

   myflash_erase_address = FLASH_TARGET_OFFSET + (sector<<12);  // each sector is 4096 bytes
   flash_range_erase(myflash_erase_address, FLASH_SECTOR_SIZE);
}
//=========================================
void test_read_sector(uint16_t sector)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_target;
   uint32_t myflash_read_address;
   char *p;

   unsigned char buffer_in[256];
   unsigned int n;

   myflash_target = FLASH_TARGET_OFFSET + (sector<<8);
   myflash_read_address = XIP_BASE + myflash_target;  
   
//   LED_break();

// read the sector
   p = (char *)myflash_read_address;
   for(n=0; n<256; n++)   { buffer_in[n] = *p++; }

   strcpy(line,"\nSector: ");
   my_itoa(sector, nstr);
   strcat(line, nstr);
   strcat(line,"  ");
   uart_puts(UART_ID, line);

   for(n=0; n<256; n++)   
   { 
      my_hextoa2(buffer_in[n], line); 
      strcat(line,"  ");   
      uart_puts(UART_ID, line);
   }
   uart_puts(UART_ID, "\n");
}
//==========================================
void test_write_sector(uint16_t sector)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256  
   
   uint32_t myflash_target;

   unsigned char buffer_out[256];
   unsigned int n;

   myflash_target = FLASH_TARGET_OFFSET + (sector<<8);            // 256 bytes per Sector;

   for(n=0; n<256; n++)  { buffer_out[n] = n; }

// write   
   flash_range_program(myflash_target, buffer_out, FLASH_PAGE_SIZE);
}
//========================================
uint16_t is_sector_blank(uint32_t sector)
{   
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data

   uint16_t n;
   uint32_t myflash_read_address;
   char *p;
   unsigned char data;
   uint16_t flag;

   flag = 1;    // blank
   myflash_read_address = XIP_BASE + (512 * 1024) + (sector<<8);
   p = (char *)myflash_read_address;
   for(n=0; n<256; n++)   
   {
      data = *p++;
      if(data != 0xFF)   
      { 
         flag = 0; 
         break;
      }
   }

   return flag;
}
//============================================
void read_sector(uint16_t sector, uint16_t number)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_target;
   uint32_t myflash_read_address;
   char *p;
   unsigned char cdata;
   unsigned int n;

   myflash_target = FLASH_TARGET_OFFSET + (sector<<8);
   myflash_read_address = XIP_BASE + myflash_target;  
   
   strcpy(line,"\nFlash Sector: ");
   my_itoa(sector, nstr);
   strcat(line, nstr);
   strcat(line,"\n");
   uart_puts(UART_ID, line);

// read the sector
   p = (char *)myflash_read_address;
   for(n=0; n<number; n++)   
   { 
      cdata = *p++;
      my_hextoa2(cdata, line); 
      strcat(line,"  ");   
      uart_puts(UART_ID, line);
   }
   uart_puts(UART_ID, "\n\n");
}
//=============================================
// Start Date is stored as a string
void do_start_date(uint16_t erase)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t buffer[256];
   uint16_t n, blank;

   // Erase if requested
   if(erase) 
   { 
     ints = save_and_disable_interrupts(); 
     myflash_erase_sector(FLASH_SDATE_SECTOR); 
     restore_interrupts(ints);
   }

//   show_page(FLASH_SDATE_PAGE);

   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_SDATE_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   

   // Is it blank?
   blank = 1;
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<256; n++)
   {
      buffer[n] = *p++;
      if( buffer[n] != 0xFF) { blank = 0;}  // not blank
   }

   if(blank)    // blank, so write today's date as a string
   { 
//      get_gps_msg_GNZDA();    // includes 'decode_time();'

      todays_date(1);   // 0 - local, 1 = utc, returns string in 'line'
      strcpy(start_date, line);  // save it  
      strcpy(line, "New Start Date: ");   
      strcat(line, start_date);
      uart_puts(UART_ID, line);

      for(n=0; n<256; n++)  {buffer[n] = 0xFF;}  // everything we don't use will remain 0xFF
      memcpy(buffer, start_date, 22);
//      strcpy(buffer, start_date);

      // write the sector   
      ints = save_and_disable_interrupts();
      flash_range_program(myflash_write_address, buffer, FLASH_PAGE_SIZE);   
      restore_interrupts(ints);
   }
   else  // not blank
   {
//      uart_puts(UART_ID,"\n We are Here 1\n\n"); 
      // read it 
      p = (uint8_t *)myflash_read_address;
  
//      for(n=0; n<32; n++)   { buffer[n] = *p++;}
      for(n=0; n<22; n++)   { start_date[n] = *p++;}
//      strcpy(start_date, "test");

//      memcpy(start_date, buffer, 32);
      strcpy(line, "Start Date: ");
      strcat(line, start_date);
//      strcat(line, "\n");

      uart_puts(UART_ID,line); 

//      uart_puts(UART_ID,"\n We are Here 2\n\n"); 
//   LED_break();
   }

/*
   // Display it in Hex
   for(n=0; n<32; n++)   
   { 
      my_hextoa2(buffer[n], line); 
      strcat(line,"  ");   
      uart_puts(UART_ID, line);
   }
   uart_puts(UART_ID, "\n\n");
*/

}  // end function
//=============================================
// make_entry(); 
// Make a Log entry to be used in RAM and Flash
void make_entry()
{
//   uint16_t n;

   struct log *ptr = &flash_entry;

   ptr->ts_number = ts_number;
   ts_number++;

   ptr->ts_date = time_buffer[DATE];
   ptr->ts_day = day_of_week;
   ptr->ts_month = time_buffer[MONTH];
   ptr->ts_year = time_buffer[YEAR];

   ptr->ts_hours = time_buffer[HOURS];
   ptr->ts_minutes = time_buffer[MINS];
   ptr->ts_seconds = time_buffer[SECS];

   ptr->flag = 0;    // flag for non-blank
}
//================================
// show_log(&flash_entry);
void show_log(struct log *ptr)
{
   uint16_t m, n;

// ts_number is the log number starting from Reset
   my_itoa(ptr->ts_number,nstr);  // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

// day name
   n = (ptr->ts_day)- 1;   // use it as an index
   for(m=0; m<3; m++)
   {
      nstr[m] = day_array[n][m];
   }
   nstr[m] = 0;
   strcat(line,nstr);
   strcat(line,"    ");   

   line_fill(ptr->ts_month, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_date, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_year, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(ptr->ts_hours, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_minutes, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_seconds, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   strcat(line,"      ");
   uart_puts(UART_ID, line); 
}
//=================================================
void write_page(uint32_t page, unsigned char *buffer)
{
   uint32_t myflash_write_address;
//   uint32_t myflash_read_address;

/*
   // Display it in Hex
   for(n=0; n<256; n++)   
   { 
      my_hextoa2(buffer[n], line); 
      strcat(line,"  ");   
      uart_puts(UART_ID, line);
   }
   uart_puts(UART_ID, "\n\n");

   return;
*/
   myflash_write_address = FLASH_TARGET_OFFSET + (page<<8); 
//   myflash_read_address = XIP_BASE + myflash_write_address;   

   // write the page   
   ints = save_and_disable_interrupts();
   flash_range_program(myflash_write_address, buffer, FLASH_PAGE_SIZE);   
   restore_interrupts(ints);

   show_page(page);
}
//=============================================
void show_log2(struct log *ptr)
{
//   uint16_t n;

   my_itoa(ptr->ts_number,nstr);  // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

   line_fill(ptr->ts_month, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_date, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_year, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(ptr->ts_hours, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_minutes, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_seconds, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   strcat(line,"      ");
   uart_puts(UART_ID, line); 
}
//========================================================
void dump_flash(uint32_t start, uint32_t end)
{
   uint32_t n;

   for(n = start; n<(end+1); n++)
   {   
      show_log_page(n);
   }
}
//========================================================
void dump_flash_page_hex(uint32_t page) 
{
   uint8_t *ptr;
   uint32_t myflash_read_address;
   uint16_t m, n;

   myflash_read_address = XIP_BASE + (512 * 1024) + (page<<8);
   ptr = (uint8_t *)myflash_read_address;

   m = 16;
   line[0] = 0;  // Start with NULL
   for(n=0; n<256; n++)
   {
      my_hextoa2(*ptr++, nstr);
      strcat(line,nstr);   

      if(--m == 0)
      {
         uart_puts(UART_ID, line); 
         m = 16;
         line[0] = 0;  // start new line
      }
      else  { strcat(line,"  "); }   
   }
   strcat(line,"\n\n");
   uart_puts(UART_ID, line); 

//LED_break();
}
//========================================================
void test10()
{   
#define PAGE 4160    // Sector is 260
   uint32_t myflash_target_address;

   uint16_t n;
   unsigned char flash_buffer[256];
   myflash_target_address = FLASH_TARGET_OFFSET + (PAGE<<8);   // 256 bytes per Page;

   ints = save_and_disable_interrupts();    // Disable Interrupts

// Erase the sector
   uart_puts(UART_ID,"Erase");
   flash_range_erase(myflash_target_address, FLASH_SECTOR_SIZE);

// Show that it is erased
   read_page(PAGE, 256);        // Read Page and send it out through uart in Hex

// Make first buffer
   for(n=0; n<256; n++) {flash_buffer[n] = 0xFF;}  // Start with all 0xFF
   for(n=0; n<32; n++) {flash_buffer[n] = n;}      // first data address 0 t0 31

// write      
   flash_range_program(myflash_target_address, flash_buffer, FLASH_PAGE_SIZE);

// Display the page
   uart_puts(UART_ID,"Bytes 0 to 31");
   read_page(PAGE, 256);        // Read Page and send it out through uart in Hex

// Make second buffer
   for(n=32; n<64; n++) {flash_buffer[n] = n;}  // second data is 32 to 63

// write      
   flash_range_program(myflash_target_address, flash_buffer, FLASH_PAGE_SIZE);

// Display the page
   uart_puts(UART_ID,"Bytes 32 to 63");
   read_page(PAGE, 256);        // Read Page and send it out through uart in Hex

// Make third buffer
   for(n=64; n<96; n++) {flash_buffer[n] = n;}  // thirst data is 64 to  95

// write      
   flash_range_program(myflash_target_address, flash_buffer, FLASH_PAGE_SIZE);

// Display the page
   uart_puts(UART_ID,"Bytes 64 to 95");
   read_page(PAGE, 256);        // Read Page and send it out through uart in Hex

// Make fourth buffer
   for(n=96; n<128; n++) {flash_buffer[n] = n;}  // thirst data is 64 to  95

// write      
   flash_range_program(myflash_target_address, flash_buffer, FLASH_PAGE_SIZE);

// Display the page
   uart_puts(UART_ID,"Bytes 96 to 127");
   read_page(PAGE, 256);        // Read Page and send it out through uart in Hex

   restore_interrupts(ints);

//   LED_break();
}
//-----------------
// copy RAM entry to flash 
void copy_ram_log_to_flash(struct log *ptr)
{
   uint16_t n;
   unsigned char flash_buffer[256];
   uint32_t myflash_target;

   memcpy(flash_buffer, ptr, structlogsize);   // ptr points to the beginning of the RAM entry

   for(n=structlogsize; n<256; n++)   { flash_buffer[n] = 0xFF; }   // fill remaining bytes in the page

// write      
   myflash_target = FLASH_TARGET_OFFSET + (Page_Index<<8);   // 256 bytes per Page;
   ints = save_and_disable_interrupts();    // Disable Interrupts
   flash_range_program(myflash_target, flash_buffer, FLASH_PAGE_SIZE);
//   restore_interrupts(ints);  // not yet

   read_page(Page_Index, 32);   // Read Page and send it out through uart in Hex
   log_flash_uart(Page_Index);  // Read and send it out formatted
   show_log_page(Page_Index);

// if Page_Index low byte is 0xFF the next page is in the next sector, so erase the next sector
// but not if this is the first time Flash has been used for entries (or has been erased)
// but if the next Sector_Index is SECTOR_MAX set it to 0  (SECTOR_MAX is 256)
   if( (Page_Index & 0xFF) == 0xFF) 
   {
      Sector_Index = (Page_Index<<4) + 1;

      if(Sector_Index >= SECTOR_MAX)  {Sector_Index = 0;}

      if(!first_time_flash)    // not the first time use of Flash so erase the next sector
      {
         myflash_erase_sector(Sector_Index);

         strcpy(line,"\nSector Erased: ");
         my_itoa(Sector_Index, nstr);
         strcat(line,nstr);
         strcat(line,"\n");
         uart_puts(UART_ID,line);
      } 
   }

   Page_Index++;   
   if(Page_Index >= 4095)  
   { 
      Page_Index = 0; 
      Last_Page = 4095;
   }

   restore_interrupts(ints);  // Enable Interrupts
}
//===================================================
void log_flash_uart(uint32_t page)
{
   uint32_t myflash_read_address;
   char *p;
   struct log flog;

   strcpy(line,"Flash Page: ");
   my_itoa(page, nstr);
   strcat(line, nstr);
   strcat(line,"\n");
   uart_puts(UART_ID, line);

   myflash_read_address = XIP_BASE + FLASH_TARGET_OFFSET + (page<<8);            // 256 bytes per Page;
   p = (char *)myflash_read_address; 
   memcpy(&flog, p, structlogsize);
   log4_data_uart(&flog);  // address of Flash 
}
//=======================
// Send a log entry out through the uart
void log4_data_uart(struct log *ptr)
{
//   uint16_t n;

   show_log_ptr(ptr);           // time stamp in 'line'
//   uart_puts(UART_ID,line);  // put out the entire line

   strcat(line,"     Data: ");
   strcat(line,"\n\n");   
   uart_puts(UART_ID,line);  // put out the entire line
}
//------------------------
// Form the time stamp number and the time for the uart  &&& From RX9
void show_log_ptr(struct log *ptr)
{
   uint16_t m, n;

// ts_number is the log number starting from Reset
   my_itoa(ptr->ts_number,nstr);  // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

   n = (ptr->ts_day)- 1;   // use it as an index
   for(m=0; m<3; m++)
   {
      nstr[m] = day_array[n][m];
   }

   nstr[m] = 0;
   strcat(line,nstr);
   strcat(line,"    ");   

   line_fill(ptr->ts_month, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_date, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_year, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(ptr->ts_hours, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_minutes, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_seconds, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");
}
//================================
// show_log_page(page);
void show_log_page(uint32_t page)
{
   uint16_t m, n;
   uint32_t myflash_read_address;
   struct log *ptr = &flash_entry; 
   char buffer[structlogsize];
   char *p;

   myflash_read_address = XIP_BASE + FLASH_TARGET_OFFSET + (page<<8);  // 256 bytes per Page;
   p = (char *)myflash_read_address;     // starts at: XIP_BASE + (512 * 1024);

//   for(n=0; n<structlogsize; n++)  { buffer[n] = *p++; }   // This is awkward but it works
//   memcpy(&flash_entry, buffer, structlogsize);
   for(n=0; n<structlogsize; n++)  { buffer[n] = *p++; }   // This is awkward but it works
   memcpy(&flash_entry, buffer, structlogsize);

// ts_number is the log number starting from Reset
   my_itoa(ptr->ts_number,nstr);  // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

// day name
   n = (ptr->ts_day)- 1;   // use it as an index
   for(m=0; m<3; m++)
   {
      nstr[m] = day_array[n][m];
   }
   nstr[m] = 0;
   strcat(line,nstr);
   strcat(line,"    ");   

   line_fill(ptr->ts_month, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_date, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(ptr->ts_year, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(ptr->ts_hours, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_minutes, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(ptr->ts_seconds, nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   strcat(line, "Temperature: ");
   temperature_itoa_str( ptr->Ftemperature, nstr);   // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," F     ");
   
   strcat(line, "Pressure: ");
   pressure_ltoa_str(ptr->Pressure, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," mb     ");

   strcat(line, "Humidity: ");
   humidity_btoa_str( ptr->Humidity, nstr);      // remove leading zeros; end with NULL
   strcat(line,nstr);
   strcat(line," %\n");
   uart_puts(UART_ID, line); 

   strcpy(line, "Barometric Pressure:  ");
   strcpy(nstr, ptr->pref_str);
   strcat(line,nstr);
   strcat(line," \n");
   uart_puts(UART_ID, line); 

// GPS
   strcpy(line, "Latitude: ");
   strcpy(nstr, ptr->gps_latitude_str);
   strcat(line,nstr);
   strcat(line,"     ");
//   uart_puts(UART_ID, line); 

   strcat(line, "Longitude: ");
   strcpy(nstr, ptr->gps_longitude_str);
   strcat(line,nstr);
   strcat(line,"     ");
//   uart_puts(UART_ID, line); 

   strcat(line, "Sats in View: ");
   strcpy(nstr, ptr->sats);
   strcat(line,nstr);
   strcat(line," \n");
   uart_puts(UART_ID, line); 

   strcpy(line, "GPS Altitude: ");
   strcpy(nstr, ptr->gps_altitude_str);
   strcat(line,nstr);
   strcat(line,"     ");

   strcat(line, "Pressure Altitude: ");
   strcpy(nstr, ptr->pressure_altitude_str);
   strcat(line,nstr);
//   strcat(line," \n");
   uart_puts(UART_ID, line); 

}
//=================================================
// If the BREF page is blank: use the value hg_bcd, then make struct pref pref_work, then write to Flash
void do_B_reference(uint16_t ref_hg_bcd)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t buffer[256];
   uint16_t n, blank;

   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_BREF_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   

   // Is it blank?
   blank = 1;
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<256; n++)
   {
      buffer[n] = *p++;
      if( buffer[n] != 0xFF) { blank = 0;}  // not blank
   }

   if(blank)    // blank, so make the structure and write it to the page
   { 
      uart_puts(UART_ID, "BREF Flash is Blank\n");
      make_pref(ref_hg_bcd);  // make structure in 'struct pref pref_work'

      for(n=0; n<256; n++)  {buffer[n] = 0xFF;}    // everything we don't use will remain 0xFF
      memcpy(buffer, &pref_work, structprefsize);  // size of pref struct (for barometric reference pressure in different formats)44);   // copies the structure
      // write the sector   
      ints = save_and_disable_interrupts();
      flash_range_program(myflash_write_address, buffer, FLASH_PAGE_SIZE);   
      restore_interrupts(ints);
   }
//LED_break();
//  show_page(FLASH_BREF_PAGE);  // hex
   uart_puts(UART_ID, "BREF Flash:");
   get_bref_flash();
   show_pref();
//LED_break();

}  // end function
//========================================================
// Barometer reference
void get_bref_flash()
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t buffer[256];
   uint16_t n;
//   uint16_t ref_hg_bcd = 0x2992;

   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_BREF_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   

   p = (uint8_t *)myflash_read_address;
   for(n=0; n<256; n++)  { buffer[n] = *p++; }
   
   memcpy(&pref_work,buffer,structprefsize);
//   show_pref();
//LED_break();

}  // end function
//====================================
void erase_bref_flash()
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   ints = save_and_disable_interrupts(); 
   myflash_erase_sector(FLASH_BREF_SECTOR); 
   restore_interrupts(ints);

//   show_page(FLASH_BREF_PAGE);
}
//===================================
// Local Page utc offset, dst enable
// If the LOCAL page is blank: use the values
void do_local(int16_t local_offset, uint16_t dst, uint16_t erase)
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t buffer[256];
   uint16_t n, blank;

   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_LOCAL_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   

   // Erase if requested
   if(erase) 
   { 
     ints = save_and_disable_interrupts(); 
     myflash_erase_sector(FLASH_LOCAL_SECTOR); 
     restore_interrupts(ints);
   }
  
   // Is it blank?
   blank = 1;
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<256; n++)
   {
      buffer[n] = *p++;
      if( buffer[n] != 0xFF) { blank = 0;}  // not blank
   }

   if(blank)    // LOCAL Flash is blank, so write value to the page
   { 
      uart_puts(UART_ID, "Local Offset Flash is Blank\n");
      for(n=0; n<256; n++)  {buffer[n] = 0xFF;}    // everything we don't use will remain 0xFF

      mylocal.utc_offset = local_offset;
      mylocal.dst_enable = dst;

      p = (uint8_t *)myflash_read_address;
      for(n=0; n<4; n++)  { buffer[n] = *p++; }
      memcpy(buffer,&mylocal, 4);

      time.utc_offset = mylocal.utc_offset;
      time.dst_observed = mylocal.dst_enable;

       // write the sector   
      ints = save_and_disable_interrupts();
      flash_range_program(myflash_write_address, buffer, FLASH_PAGE_SIZE);   
      restore_interrupts(ints);
   }

   p = (uint8_t *)myflash_read_address;
   time.utc_offset = *p++;  // global
   time.dst_observed = *p++;

   strcpy(line,"\nLOCAL Flash:\nLocal Offset: ");
   sint8_bcd_str(time.utc_offset, nstr);    // signed char to ASCII string, 3 digits
   strcat(line, nstr);
   strcat(line, " Hours    DST: ");
   if(time.dst_observed)  { strcat(line,"Observed\n\n"); }
   else { strcat(line,"Not Enabled\n\n"); }
   uart_puts(UART_ID, line);
//LED_break();
}  // end function
//========================================================
// Local Page utc offset, dst enable
void get_local_flash()
{
// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t buffer[16];
   int16_t n;

   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_LOCAL_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<4; n++)  { buffer[n] = *p++; }
   memcpy(&mylocal, buffer, 4);
   time.utc_offset = mylocal.utc_offset;
   time.dst_observed = mylocal.dst_enable;
//test_sint16(20, time.utc_offset);
//test_sint16(21, time.dst_observed);
//LED_break();

}  // end function
//====================================
// dst start date and end date
void do_dst(uint16_t erase)
{
// Is dst flash blank?

// #define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
// FLASH_PAGE_SIZE is 256    // already defined
   
   uint32_t myflash_write_address;
   uint32_t myflash_read_address;
   uint8_t *p;
   uint8_t buffer[256];
   uint16_t n, blank;

   // Erase if requested
   if(erase) 
   { 
     ints = save_and_disable_interrupts(); 
     myflash_erase_sector(FLASH_DST_SECTOR); 
     restore_interrupts(ints);
   }

   myflash_write_address = FLASH_TARGET_OFFSET + (FLASH_DST_PAGE<<8); 
   myflash_read_address = XIP_BASE + myflash_write_address;   

   // Is it blank?
   blank = 1;
   p = (uint8_t *)myflash_read_address;
   for(n=0; n<256; n++)
   {
      buffer[n] = *p++;
      if( buffer[n] != 0xFF) { blank = 0;}  // not blank
   }

   if(blank)    // blank, so get dst start and end dates
   { 
        // first get the current utc year
//        get_gps_msg_GNZDA();    // time (year)

        strcpy(line, "Doing DST, utc year is: ");
        uint16_bcd_str4(time.utc_hex_year, nstr);    // uint16 to ASCII string, 4 digits 
        strcat(line, nstr);
        strcat(line, "\n\n");
        uart_puts(UART_ID, line);

       // get start and end dates for dst
   }

/*
// Display Start Date in Hex
      uart_puts(UART_ID, "\nTime Hex Data: ");
      for(n=0; n<8; n++)   
      { 
         my_hextoa2(time_buffer[n], line); 
         strcat(line,"  ");   
         uart_puts(UART_ID, line);
      }
      uart_puts(UART_ID, "\n\n");
*/
}  // end function
//========================================================
void todays_date(uint16_t utc)
{
//   get_gps_msg_GNZDA();    // time

   if(utc)
   {
      strcpy(line, month_name[time.utc_hex_month - 1 ]);  
      strcat(line," ");

      strcat(line,time.utc_gps_date);
      strcat(line,", ");

      strcat(line,time.utc_gps_year);
      strcat(line,"\n\n");
   }
   else   // local date as a string
   {
      // test
      time.local_hex_month = 9;
      time.local_hex_date = 22;
      time.local_hex_year = 2024;

      strcpy(line, month_name[time.local_hex_month - 1 ]);  
      strcat(line," ");

      uint8_bcd_str2(time.local_hex_date, nstr);  // unsigned char to ASCII string, 2 digits   
      strcat(line,nstr);
      strcat(line,", ");

      uint16_bcd_str4(time.local_hex_year, nstr);   // uint16 to ASCII string, 4 digits
      strcat(line,nstr);
      strcat(line,"\n\n");
   }

//   uart_puts(UART_ID, line);
}
//=============================================
